--------------------------------------------------------------------------
-- LIBRARY INCLUDES
--------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

--------------------------------------------------------------------------
-- SIMULATION LIBRARY INCLUDES
--------------------------------------------------------------------------
-- pragma translate_off

library UNISIM;
use unisim.all;

-- pragma translate_on

--------------------------------------------------------------------------
-- PORT DECLARATION
--------------------------------------------------------------------------

entity bram is
  generic (
    g_data_width : integer := 1
  );
  port (
    IN_clk    : in  std_logic; -- clk40 input
    IN_rst    : in  std_logic; -- asynchronous global reset
    IN_data   : in  std_logic_vector(g_data_width-1 downto 0);
    IN_wr_adr : in  std_logic_vector(11 downto  0); -- 
    IN_rd_adr : in  std_logic_vector(11 downto  0); --
    IN_wren   : in  std_logic; --
    OUT_data  : out std_logic_vector(g_data_width-1 downto 0)
    );
end bram;

--------------------------------------------------------------------------
-- XILINX PARTS SPECIFIC COMPONENTS/SIGNALS
--------------------------------------------------------------------------

--------------------------------------------------------------------------
-- SIGNAL DECLARATION
--------------------------------------------------------------------------
architecture behave of bram is


--------------------------------------------------------------------------
-- COMPONENT DECLARATION
--------------------------------------------------------------------------

component RAMB4_S1_S1
  port (
    DOA   : out STD_LOGIC_VECTOR( 0 DOWNTO 0);
    DOB   : out STD_LOGIC_VECTOR( 0 DOWNTO 0);
    ADDRA : in  STD_LOGIC_VECTOR(11 DOWNTO 0);
    DIA   : in  STD_LOGIC_VECTOR( 0 DOWNTO 0);
    ENA   : in  STD_ULOGIC;
    CLKA  : in  STD_ULOGIC;
    WEA   : in  STD_ULOGIC;                
    RSTA  : in  STD_ULOGIC;
    ADDRB : in  STD_LOGIC_VECTOR(11 DOWNTO 0);
    DIB   : in  STD_LOGIC_VECTOR( 0 DOWNTO 0);
    ENB   : in  STD_ULOGIC;
    CLKB  : in  STD_ULOGIC;
    WEB   : in  STD_ULOGIC;
    RSTB  : in  STD_ULOGIC
    ); 
end component;

component RAMB4_S2_S2
  port (
    DOA   : out STD_LOGIC_VECTOR( 1 DOWNTO 0);
    DOB   : out STD_LOGIC_VECTOR( 1 DOWNTO 0);
    ADDRA : in  STD_LOGIC_VECTOR(10 DOWNTO 0);
    DIA   : in  STD_LOGIC_VECTOR( 1 DOWNTO 0);
    ENA   : in  STD_ULOGIC;
    CLKA  : in  STD_ULOGIC;
    WEA   : in  STD_ULOGIC;                
    RSTA  : in  STD_ULOGIC;
    ADDRB : in  STD_LOGIC_VECTOR(10 DOWNTO 0);
    DIB   : in  STD_LOGIC_VECTOR( 1 DOWNTO 0);
    ENB   : in  STD_ULOGIC;
    CLKB  : in  STD_ULOGIC;
    WEB   : in  STD_ULOGIC;
    RSTB  : in  STD_ULOGIC
    ); 
end component;

component RAMB4_S4_S4
  port (
    DOA   : out STD_LOGIC_VECTOR( 3 DOWNTO 0);
    DOB   : out STD_LOGIC_VECTOR( 3 DOWNTO 0);
    ADDRA : in  STD_LOGIC_VECTOR( 9 DOWNTO 0);
    DIA   : in  STD_LOGIC_VECTOR( 3 DOWNTO 0);
    ENA   : in  STD_ULOGIC;
    CLKA  : in  STD_ULOGIC;
    WEA   : in  STD_ULOGIC;                
    RSTA  : in  STD_ULOGIC;
    ADDRB : in  STD_LOGIC_VECTOR( 9 DOWNTO 0);
    DIB   : in  STD_LOGIC_VECTOR( 3 DOWNTO 0);
    ENB   : in  STD_ULOGIC;
    CLKB  : in  STD_ULOGIC;
    WEB   : in  STD_ULOGIC;
    RSTB  : in  STD_ULOGIC
    ); 
end component;

component RAMB4_S8_S8
  port (
    DOA   : out STD_LOGIC_VECTOR( 7 DOWNTO 0);
    DOB   : out STD_LOGIC_VECTOR( 7 DOWNTO 0);
    ADDRA : in  STD_LOGIC_VECTOR( 8 DOWNTO 0);
    DIA   : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    ENA   : in  STD_ULOGIC;
    CLKA  : in  STD_ULOGIC;
    WEA   : in  STD_ULOGIC;                
    RSTA  : in  STD_ULOGIC;
    ADDRB : in  STD_LOGIC_VECTOR( 8 DOWNTO 0);
    DIB   : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    ENB   : in  STD_ULOGIC;
    CLKB  : in  STD_ULOGIC;
    WEB   : in  STD_ULOGIC;
    RSTB  : in  STD_ULOGIC
    ); 
end component;

component RAMB4_S16_S16
  port (
    DOA   : out STD_LOGIC_VECTOR(15 DOWNTO 0);
    DOB   : out STD_LOGIC_VECTOR(15 DOWNTO 0);
    ADDRA : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    DIA   : in  STD_LOGIC_VECTOR(15 DOWNTO 0);
    ENA   : in  STD_ULOGIC;
    CLKA  : in  STD_ULOGIC;
    WEA   : in  STD_ULOGIC;                
    RSTA  : in  STD_ULOGIC;
    ADDRB : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    DIB   : in  STD_LOGIC_VECTOR(15 DOWNTO 0);
    ENB   : in  STD_ULOGIC;
    CLKB  : in  STD_ULOGIC;
    WEB   : in  STD_ULOGIC;
    RSTB  : in  STD_ULOGIC
    ); 
end component;


--attribute INIT_00: string;
--attribute INIT_01: string;
--attribute INIT_02: string;
--attribute INIT_03: string;
--attribute INIT_04: string;
--attribute INIT_05: string;
--attribute INIT_06: string;
--attribute INIT_07: string;
--attribute INIT_08: string;
--attribute INIT_09: string;
--attribute INIT_0A: string;
--attribute INIT_0B: string;
--attribute INIT_0C: string;
--attribute INIT_0D: string;
--attribute INIT_0E: string;
--attribute INIT_0F: string;
--
--attribute INIT_00 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_01 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_02 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_03 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_04 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_05 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_06 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_07 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_08 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_09 of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_0A of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_0B of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_0C of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_0D of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_0E of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";
--attribute INIT_0F of MB: label is
--"0000000000000000000000000000000000000000000000000000000000000000";


-------------------------------------------------------------------------------
-- COMPONENT INSTANTIATION
-------------------------------------------------------------------------------
begin

gen_width_1 : if g_data_width = 1 generate
	MB : RAMB4_S1_S1
	  port map(
		 DOA   => open,
		 DOB   => OUT_data,
		 ADDRA => IN_wr_adr(11 downto 0),
		 DIA   => IN_data,
		 ENA   => '1',
		 CLKA  => IN_clk,
		 WEA   => IN_wren,
		 RSTA  => IN_rst,
		 ADDRB => IN_rd_adr(11 downto 0),
		 DIB   => "0",
		 ENB   => '1',
		 CLKB  => IN_clk,
		 WEB   => '0',
		 RSTB  => IN_rst
    ); 
end generate;

gen_width_2 : if g_data_width = 2 generate
	MB : RAMB4_S2_S2
	  port map(
		 DOA   => open,
		 DOB   => OUT_data,
		 ADDRA => IN_wr_adr(10 downto 0),
		 DIA   => IN_data,
		 ENA   => '1',
		 CLKA  => IN_clk,
		 WEA   => IN_wren,
		 RSTA  => IN_rst,
		 ADDRB => IN_rd_adr(10 downto 0),
		 DIB   => "00",
		 ENB   => '1',
		 CLKB  => IN_clk,
		 WEB   => '0',
		 RSTB  => IN_rst
		 ); 
end generate;

gen_width_4 : if g_data_width = 4 generate
	MB : RAMB4_S4_S4
	  port map(
		 DOA   => open,
		 DOB   => OUT_data,
		 ADDRA => IN_wr_adr(9 downto 0),
		 DIA   => IN_data,
		 ENA   => '1',
		 CLKA  => IN_clk,
		 WEA   => IN_wren,
		 RSTA  => IN_rst,
		 ADDRB => IN_rd_adr(9 downto 0),
		 DIB   => X"0",
		 ENB   => '1',
		 CLKB  => IN_clk,
		 WEB   => '0',
		 RSTB  => IN_rst
		 ); 
end generate;

gen_width_8 : if g_data_width = 8 generate
	MB : RAMB4_S8_S8
	  port map(
		 DOA   => open,
		 DOB   => OUT_data,
		 ADDRA => IN_wr_adr(8 downto 0),
		 DIA   => IN_data,
		 ENA   => '1',
		 CLKA  => IN_clk,
		 WEA   => IN_wren,
		 RSTA  => IN_rst,
		 ADDRB => IN_rd_adr(8 downto 0),
		 DIB   => X"00",
		 ENB   => '1',
		 CLKB  => IN_clk,
		 WEB   => '0',
		 RSTB  => IN_rst
		 ); 
end generate;

gen_width_16 : if g_data_width = 16 generate
	MB : RAMB4_S16_S16
	  port map(
		 DOA   => open,
		 DOB   => OUT_data,
		 ADDRA => IN_wr_adr(7 downto 0),
		 DIA   => IN_data,
		 ENA   => '1',
		 CLKA  => IN_clk,
		 WEA   => IN_wren,
		 RSTA  => IN_rst,
		 ADDRB => IN_rd_adr(7 downto 0),
		 DIB   => X"0000",
		 ENB   => '1',
		 CLKB  => IN_clk,
		 WEB   => '0',
		 RSTB  => IN_rst
		 ); 
end generate;

end behave;
