----------------------------------------------------------------------------
-- LBL - WISCONSIN
-- ATLAS ROD ELECTRONICS
----------------------------------------------------------------------------
-- Filename:
--    Event Fragment Builder: top.vhd
-- Description:
--    ROD Dual Gatherer
--    SCT/Pixel Version
--
-- Notes:
------------------------------------------------------------------------------
--
----------------------------------------------------------------------------

library IEEE ;
use IEEE.STD_LOGIC_1164.all ;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- pragma translate_off
library UNISIM;
use unisim.all;
-- pragma translate_on

----------------------------------------------------------------------------
--PORT DECLARATION
----------------------------------------------------------------------------

ENTITY efb_top IS
  port (
    clock                  : in  STD_LOGIC ;
    reset                  : in  STD_LOGIC ;

-- FIFO Controller Signals
    data_in1               : in  STD_LOGIC_VECTOR(39 DOWNTO 0) ;
    data_in2               : in  STD_LOGIC_VECTOR(39 DOWNTO 0) ;
    data_valid_in_n1       : in  STD_LOGIC_VECTOR(3  DOWNTO 0) ;
    data_valid_in_n2       : in  STD_LOGIC_VECTOR(3  DOWNTO 0) ;
    form_enabled_n1        : in  STD_LOGIC_VECTOR(3  DOWNTO 0) ;
    form_enabled_n2        : in  STD_LOGIC_VECTOR(3  DOWNTO 0) ;
    fifo1_pause            : out STD_LOGIC ;
    fifo2_pause            : out STD_LOGIC ;

-- EFB Bus Interface Signals
    efb_bus_sel_n          : in    STD_LOGIC ;
    efb_bus_rnw            : in    STD_LOGIC ;
    efb_bus_addr           : in    STD_LOGIC_VECTOR(7 DOWNTO 0) ;
    efb_bus_data           : inout STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    efb_bus_ack            : out   STD_LOGIC ;

-- Event Data L1/BCID/Trig Type Data
    ev_data                : in  STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    ev_data_wen_n          : in  STD_LOGIC ;
    ev_data_rst_n          : in  STD_LOGIC ;  -- jmj 20000626
    ev_data_almost_full_n  : out STD_LOGIC ;
    ev_data_empty          : out STD_LOGIC ;
    ev_id_fifo_empty_error : out STD_LOGIC ;

-- Output / SLink / Back End DSP Signals
    gatherer_halt_output   : in  STD_LOGIC ;
    out_fifo1_empty_n      : in  STD_LOGIC ;
    out_fifo1_aempty_n     : in  STD_LOGIC ;
    out_fifo1_full_n       : in  STD_LOGIC ;
    out_fifo1_afull_n      : in  STD_LOGIC ;
    out_fifo1_data         : out STD_LOGIC_VECTOR(45 DOWNTO 0) ;
    out_fifo1_wen_n        : out STD_LOGIC ;
    out_fifo1_rst_n        : out STD_LOGIC ;
    out_fifo1_ren_n        : out STD_LOGIC ;
    out_fifo1_oe_n         : out STD_LOGIC ;
    out_fifo2_empty_n      : in  STD_LOGIC ;
    out_fifo2_aempty_n     : in  STD_LOGIC ;
    out_fifo2_full_n       : in  STD_LOGIC ;
    out_fifo2_afull_n      : in  STD_LOGIC ;
    out_fifo2_data         : out STD_LOGIC_VECTOR(45 DOWNTO 0) ;
    out_fifo2_wen_n        : out STD_LOGIC ;
    out_fifo2_rst_n        : out STD_LOGIC ;
    out_fifo2_ren_n        : out STD_LOGIC ;
    out_fifo2_oe_n         : out STD_LOGIC ;
    ev_header_data         : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    tri_state_out_fifo1    : out STD_LOGIC ;
    tri_state_out_fifo2    : out STD_LOGIC ;
    tri_state_ev_header    : out STD_LOGIC ;
    data_valid_out         : out STD_LOGIC ;
    
    rod_type_in            : in  STD_LOGIC ;  -- '0'=>SCT, '1'=>Pixel
    eventmem_fifo_cntrl    : in  STD_LOGIC_VECTOR(7 downto 0) ;
    link_data_mux_in       : in  STD_LOGIC_VECTOR(7 downto 0) ;
    link_data_mux_out      : out STD_LOGIC ;
    test_out               : out STD_LOGIC_VECTOR(7 downto 0) ;
    spare_pin              : inout STD_LOGIC_VECTOR(23 downto 0)
    );
END efb_top ;

-- eventmem_fifo_cntrl(7:0) bit definitions:
--
--  bit    signal description
--   0     efb_a_re_n    :  efb_a fifo read enable (active low)
--   1     efb_a_oe_n    :  efb_a fifo output enable (active low)
--   2     efb_a_en_n    :  efb_a fifo bus enable (active low)
--   3     efb_b_re_n    :  efb_b fifo read enable (active low)
--   4     efb_b_oe_n    :  efb_b fifo output enable (active low)
--   5     efb_b_en_n    :  efb_b fifo bus enable (active low)
--   6     efb_htf_re_n  :  header fifo read enable (active low)
--   7     efb_htf_en_n  :  header fifo bus enable (active low)

architecture rtl of efb_top is

----------------------------------------------------------------------------
--SIGNAL DECLARATION
----------------------------------------------------------------------------

  signal gnd                         : STD_LOGIC;
  signal pwr                         : STD_LOGIC;

  signal halt_output                 : STD_LOGIC;

  signal ev_id_data_in1              : STD_LOGIC_VECTOR(111 DOWNTO 0); 
  signal ev_id_data_out1             : STD_LOGIC_VECTOR(111 DOWNTO 0); 
  signal ev_id_ren1                  : STD_LOGIC;
  signal ev_id_wen1                  : STD_LOGIC;
  signal ev_id_full1                 : STD_LOGIC;
  signal ev_id_empty1                : STD_LOGIC;
  signal ev_id_fifo_empty_error1     : STD_LOGIC;
  signal ev_id_almost_full1          : STD_LOGIC;

  signal ev_id_data_in2              : STD_LOGIC_VECTOR(111 DOWNTO 0);   
  signal ev_id_data_out2             : STD_LOGIC_VECTOR(111 DOWNTO 0); 
  signal ev_id_ren2                  : STD_LOGIC;
  signal ev_id_wen2                  : STD_LOGIC;
  signal ev_id_full2                 : STD_LOGIC;
  signal ev_id_empty2                : STD_LOGIC;
  signal ev_id_fifo_empty_error2     : STD_LOGIC;
  signal ev_id_almost_full2          : STD_LOGIC;

  signal header_ev_id_data_in        : STD_LOGIC_VECTOR(63 DOWNTO 0); 
  signal header_ev_id_data_out       : STD_LOGIC_VECTOR(63 DOWNTO 0); 
  signal header_ev_id_ren            : STD_LOGIC;
  signal header_ev_id_wen            : STD_LOGIC;
  signal header_ev_id_full           : STD_LOGIC;
  signal header_ev_id_empty          : STD_LOGIC;
  signal header_ev_id_almost_full    : STD_LOGIC;
  signal header_ev_id_data_stts_reg  : STD_LOGIC_VECTOR(15 DOWNTO 0); 
  
  signal eventmem_fifo_cntrl_i       : STD_LOGIC_VECTOR(7 downto 0);  -- jmj 20000626
  signal eventmem_fifo_cntrl_ii      : STD_LOGIC_VECTOR(7 downto 0);  -- jmj 20000626

  signal gatherer_id1                : STD_LOGIC;
  signal id_check1_data_out          : STD_LOGIC_VECTOR(46 DOWNTO 0); 
  signal id_check1_data_valid_out    : STD_LOGIC;
    
  signal gatherer_id2                : STD_LOGIC;
  signal id_check2_data_out          : STD_LOGIC_VECTOR(46 DOWNTO 0); 
  signal id_check2_data_valid_out    : STD_LOGIC;

  signal err_mask_wen1               : STD_LOGIC ;
  signal err_mask_din1               : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal err_mask_dout1              : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal err_mask_sel1               : STD_LOGIC_VECTOR( 5 DOWNTO 0);      

  signal err_det1_data_out           : STD_LOGIC_VECTOR(45 DOWNTO 0);
  signal err_det1_data_valid_out     : STD_LOGIC ;

  signal err_sumry_fifo1_wen         : STD_LOGIC ;
  signal err_sumry_fifo1_data_in     : STD_LOGIC_VECTOR(47 DOWNTO 0);
  signal err_sumry_fifo1_data_out    : STD_LOGIC_VECTOR(47 DOWNTO 0);
  signal err_sumry_fifo1_ren         : STD_LOGIC ;
  signal err_sumry_fifo1_full        : STD_LOGIC ;
  signal err_sumry_fifo1_empty       : STD_LOGIC ;
  signal err_sumry_fifo1_almost_full : STD_LOGIC ;

  signal err_mask_wen2               : STD_LOGIC ;
  signal err_mask_din2               : STD_LOGIC_VECTOR(31 DOWNTO 0)  ;
  signal err_mask_dout2              : STD_LOGIC_VECTOR(31 DOWNTO 0)  ;
  signal err_mask_sel2               : STD_LOGIC_VECTOR( 5 DOWNTO 0)  ;      
          
  signal err_det2_data_out           : STD_LOGIC_VECTOR(45 DOWNTO 0) ;
  signal err_det2_data_valid_out     : STD_LOGIC ;   
  signal err_sumry_fifo2_wen         : STD_LOGIC ;
  signal err_sumry_fifo2_data_in     : STD_LOGIC_VECTOR(47 DOWNTO 0)  ;
  signal err_sumry_fifo2_data_out    : STD_LOGIC_VECTOR(47 DOWNTO 0)  ;
  signal err_sumry_fifo2_ren         : STD_LOGIC ;
  signal err_sumry_fifo2_full        : STD_LOGIC ;
  signal err_sumry_fifo2_empty       : STD_LOGIC ;
  signal err_sumry_fifo2_almost_full : STD_LOGIC ;

  signal count_fifo1_data_in         : STD_LOGIC_VECTOR(15 DOWNTO 0)  ;
  signal count_fifo1_data_out        : STD_LOGIC_VECTOR(15 DOWNTO 0)  ;
  signal count_fifo1_ren             : STD_LOGIC ;
  signal count_fifo1_wen             : STD_LOGIC ;
  signal count_fifo1_full            : STD_LOGIC ;
  signal count_fifo1_empty           : STD_LOGIC ;
  signal count_fifo1_almost_full     : STD_LOGIC ;
  
  signal count_fifo2_data_in         : STD_LOGIC_VECTOR(15 DOWNTO 0)  ;
  signal count_fifo2_data_out        : STD_LOGIC_VECTOR(15 DOWNTO 0)  ;
  signal count_fifo2_ren             : STD_LOGIC ;
  signal count_fifo2_wen             : STD_LOGIC ;
  signal count_fifo2_full            : STD_LOGIC ;
  signal count_fifo2_empty           : STD_LOGIC ;
  signal count_fifo2_almost_full     : STD_LOGIC ;

  signal out_mem1_play               : STD_LOGIC ;
  signal out_mem1_num_words          : STD_LOGIC_VECTOR(13 DOWNTO 0) ;
  signal out_mem1_done               : STD_LOGIC ;
  signal out_mem1_data_valid         : STD_LOGIC ;
  signal out_mem2_play               : STD_LOGIC ;
  signal out_mem2_num_words          : STD_LOGIC_VECTOR(13 DOWNTO 0) ;
  signal out_mem2_done               : STD_LOGIC ;
  signal out_mem2_data_valid         : STD_LOGIC ;
  signal ev_header_data_valid_out    : STD_LOGIC ;
  signal ev_header_data_valid_out_i  : STD_LOGIC ;
  signal ev_header_data_valid_out_d1 : STD_LOGIC ;

  signal out_mem1_play_i             : STD_LOGIC ;
  signal out_mem1_num_words_i        : STD_LOGIC_VECTOR(13 DOWNTO 0) ;
  signal out_mem2_play_i             : STD_LOGIC ;
  signal out_mem2_num_words_i        : STD_LOGIC_VECTOR(13 DOWNTO 0) ;

  signal out_fifo1_oe_n_p1           : STD_LOGIC ;
  signal out_fifo2_oe_n_p1           : STD_LOGIC ;  

  signal run_number                  : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal source_id                   : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal format_version              : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal send_empty_events           : STD_LOGIC ; 
  signal rst_or_flush                : STD_LOGIC ; 

  signal out_fifo_diag_mode          : STD_LOGIC_VECTOR(2 DOWNTO 0)  ;
  signal out_fifo1_diag_en           : STD_LOGIC ;
  signal out_fifo2_diag_en           : STD_LOGIC ;
  signal ht_fifo_diag_en             : STD_LOGIC ;
  signal out_fifo_rst                : STD_LOGIC ;
  signal out_fifo_data_word          : STD_LOGIC_VECTOR(45 DOWNTO 0);
  signal out_fifo_write_data         : STD_LOGIC ;
  signal out_fifo1_num_words_play    : STD_LOGIC_VECTOR(13 DOWNTO 0) ;
  signal out_fifo2_num_words_play    : STD_LOGIC_VECTOR(13 DOWNTO 0) ;
  signal out_fifo_play_data          : STD_LOGIC ;

  signal clk                         : STD_LOGIC ;
  signal clk0                        : STD_LOGIC ;
  signal clk90                       : STD_LOGIC ;
  signal clk270                      : STD_LOGIC ;
  signal clock_in                    : STD_LOGIC ;
  signal dll_clk0_out                : STD_LOGIC ;
  signal dll_clk90_out               : STD_LOGIC ;
  signal dll_clk270_out              : STD_LOGIC ;
  signal dll_locked_i                : STD_LOGIC ;
 
--  divide attribute for main_clk_dll / divide by 8 (5 MHz | 200ns)
--  attribute CLKDV_DIVIDE : string;
--  attribute CLKDV_DIVIDE of main_clk_dll : label is "8";

  signal rst                         : STD_LOGIC ;
  signal rst_n                       : STD_LOGIC ;    -- jmj 20000623
  signal data_in1_i                  : STD_LOGIC_VECTOR(39 DOWNTO 0) ;
  signal data_in2_i                  : STD_LOGIC_VECTOR(39 DOWNTO 0) ;
  signal data_valid_in_n1_i          : STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
  signal data_valid_in_n2_i          : STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
  signal form_enabled_n1_i           : STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
  signal form_enabled_n2_i           : STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
  signal fifo1_pause_o               : STD_LOGIC ;
  signal fifo2_pause_o               : STD_LOGIC ;
  signal efb_bus_sel_n_i             : STD_LOGIC ;
  signal efb_bus_rnw_i               : STD_LOGIC ;
  signal efb_bus_addr_i              : STD_LOGIC_VECTOR( 7 DOWNTO 0) ;
  signal efb_bus_data_in             : STD_LOGIC_VECTOR(15 DOWNTO 0) ;
  signal efb_bus_data_out            : STD_LOGIC_VECTOR(15 DOWNTO 0) ;
  signal efb_bus_data_oe_n           : STD_LOGIC ;
  signal efb_bus_ack_o               : STD_LOGIC ;
  signal ev_data_i                   : STD_LOGIC_VECTOR(15 DOWNTO 0) ;
  signal ev_data_ii                  : STD_LOGIC_VECTOR(15 DOWNTO 0) ;
  signal ev_data_wen_n_i             : STD_LOGIC ;
  signal ev_data_rst_n_i             : STD_LOGIC ;
  signal ev_data_flush               : STD_LOGIC ;
  signal ev_data_almost_full_n_o     : STD_LOGIC ;
  signal ev_data_almost_full_n_p     : STD_LOGIC ;
  signal ev_data_empty_o             : STD_LOGIC ;

  signal out_fifo1_data_i            : STD_LOGIC_VECTOR(45 DOWNTO 0) ;
  signal out_fifo1_wen_n_i           : STD_LOGIC ;
  signal out_fifo1_ren_n_i           : STD_LOGIC ;
  signal out_fifo2_data_i            : STD_LOGIC_VECTOR(45 DOWNTO 0) ;
  signal out_fifo2_wen_n_i           : STD_LOGIC ;
  signal out_fifo2_ren_n_i           : STD_LOGIC ;

  signal gatherer_halt_output_i      : STD_LOGIC ;
  signal out_fifo1_empty_n_i         : STD_LOGIC ;
  signal out_fifo1_aempty_n_i        : STD_LOGIC ;
  signal out_fifo1_full_n_i          : STD_LOGIC ;
  signal out_fifo1_afull_n_i         : STD_LOGIC ;
  signal out_fifo1_data_o            : STD_LOGIC_VECTOR(45 DOWNTO 0) ;
  signal out_fifo1_wen_n_o           : STD_LOGIC ;
  signal out_fifo1_ren_n_o           : STD_LOGIC ;  
  signal out_fifo1_oe_n_o            : STD_LOGIC ;
  signal out_fifo1_rst_n_o           : STD_LOGIC ;  

  signal out_fifo2_empty_n_i         : STD_LOGIC ;
  signal out_fifo2_aempty_n_i        : STD_LOGIC ;
  signal out_fifo2_full_n_i          : STD_LOGIC ;
  signal out_fifo2_afull_n_i         : STD_LOGIC ;
  signal out_fifo2_data_o            : STD_LOGIC_VECTOR(45 DOWNTO 0) ;
  signal out_fifo2_wen_n_o           : STD_LOGIC ;
  signal out_fifo2_ren_n_o           : STD_LOGIC ;
  signal out_fifo2_oe_n_o            : STD_LOGIC ;
  signal out_fifo2_rst_n_o           : STD_LOGIC ;

  signal ev_header_data_o            : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal ev_header_data_i            : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal tri_state_out_fifo1_o       : STD_LOGIC ;
  signal tri_state_out_fifo2_o       : STD_LOGIC ;
  signal tri_state_ev_header_o       : STD_LOGIC ;
  signal data_valid_out_o            : STD_LOGIC ;
  signal ev_id_fifo_empty_error_o    : STD_LOGIC ;

  signal ht_fifo_we_i          : STD_LOGIC ;
  signal ht_fifo_re_i          : STD_LOGIC ;
  signal ht_fifo_empty_i       : STD_LOGIC ;
  signal ht_fifo_full_i        : STD_LOGIC ;
  signal en_ht_fifo_out_i      : STD_LOGIC ;
  signal en_ht_fifo_out_d1     : STD_LOGIC ;
  signal htf_debug_data_out_i  : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal htf_debug_data_in_i   : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal efb_misc_stts_reg_i   : STD_LOGIC_VECTOR(30 DOWNTO 0) ;
  signal eventmem_status_reg_i : STD_LOGIC_VECTOR( 7 DOWNTO 0) ;

  signal efb_ready_i        : STD_LOGIC ;
  signal bcid_offset        : STD_LOGIC_VECTOR( 7 DOWNTO 0);
  signal mask_bcid_error    : STD_LOGIC ;
  signal mask_l1id_error    : STD_LOGIC ;
  signal mask_empty_error   : STD_LOGIC ;
  signal rod_type_o         : STD_LOGIC ;
  signal link_in_mux_i      : STD_LOGIC_VECTOR(3 downto 0) ; 
  signal link_data_o        : STD_LOGIC ;
  
  signal spare_i            : STD_LOGIC_VECTOR(23 DOWNTO 0) ;   
  signal spare_o            : STD_LOGIC_VECTOR(23 DOWNTO 0) ;   
  signal spare_t            : STD_LOGIC_VECTOR(23 DOWNTO 0) ;   
  signal link_mux_test_p    : STD_LOGIC_VECTOR( 7 DOWNTO 0) ;

  signal event_count_i      : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal grp_evt_count1     : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal grp_evt_count2     : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal grp_evt_enable     : STD_LOGIC ;

  signal bandwidth_status_i : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
  signal time_count_i       : STD_LOGIC_VECTOR(15 DOWNTO 0) ;
  signal bw_count_i         : STD_LOGIC_VECTOR(15 DOWNTO 0) ;

  signal mask_boc_clock_err_i : STD_LOGIC;
  signal mask_tim_clock_err_i : STD_LOGIC;
  signal sweeper_event_i      : STD_LOGIC;
  signal sync_event_error_i   : STD_LOGIC;
  signal hold_next_event_i    : STD_LOGIC;

  signal l1id_diag_data : STD_LOGIC_VECTOR(31 downto 0);
  signal l1id_diag_addr : STD_LOGIC_VECTOR( 7 downto 0);
  signal enable_l1_trap : STD_LOGIC;
  signal l1_trap_full1  : STD_LOGIC;
  signal l1_trap_full2  : STD_LOGIC;
  signal l1_trap_full   : STD_LOGIC;
  signal latched_l1id0  : STD_LOGIC_VECTOR(3 downto 0);
  signal latched_l1id1  : STD_LOGIC_VECTOR(3 downto 0);
  signal latched_bcid0  : STD_LOGIC_VECTOR(7 downto 0);
  signal latched_bcid1  : STD_LOGIC_VECTOR(7 downto 0);
  
  signal tim_bcid_in_rol_i       : STD_LOGIC;
  signal bcid_rollover_select    : STD_LOGIC;
  signal hv_flag_set_i           : STD_LOGIC_VECTOR( 3 downto 0);
  signal event_threshold_limit_i : STD_LOGIC_VECTOR(15 downto 0);
  signal evid_fifo_count_i       : STD_LOGIC_VECTOR( 7 downto 0);
  signal count_fifo1_wc          : STD_LOGIC_VECTOR( 7 downto 0);
  signal count_fifo2_wc          : STD_LOGIC_VECTOR( 7 downto 0);
  signal evtmem_fifo_wc_stts_i   : STD_LOGIC_VECTOR(21 downto 0);
  
  signal halt_delay_count : unsigned(5 downto 0);
  signal xoff_pipeline : STD_LOGIC_VECTOR(3 downto 0);
  type xoff_state_typedef is (
    wait_for_xoff,
    halt_efb
    );
  signal xoff_control_state : xoff_state_typedef;
    
signal diag_fifo_control    : std_logic_vector(31 downto 0);
signal diag_fifo_status     : std_logic_vector(31 downto 0);
signal diag_fifo_trig_mask  : std_logic_vector(31 downto 0); 
signal diag_fifo_triggers   : std_logic_vector(31 downto 0); 
signal diag_fifo_data_i     : std_logic_vector(31 downto 0); 
signal diag_fifo_valid_i    : std_logic; 
signal diag_fifo_data_o     : std_logic_vector(31 downto 0); 

  signal finish_frag_after_ECR   : STD_LOGIC;
  signal announce_ecr            : STD_LOGIC;
  signal announce_ecr_enable     : std_logic;
  
  signal l1id_err_data           : std_logic_vector(31 downto 0);
  signal bcid_err_data           : std_logic_vector(31 downto 0);
  signal desync_cnter_addr       : std_logic_vector( 3 downto 0);
  signal desync_cnter_rst        : std_logic;
  
  signal id_check1_l1id_old      : std_logic;
  signal id_check1_bcid_old      : std_logic;
  signal id_check2_l1id_old      : std_logic;
  signal id_check2_bcid_old      : std_logic;
  
  type t_counterArray is array (0 to 15) of std_logic_vector(31 downto 0);
  signal l1id_err_cnter          : t_counterArray;
  signal bcid_err_cnter          : t_counterArray;

--------------------------------------------------------------------------
-- XILINX PARTS SPECIFIC COMPONENTS/SIGNALS
--------------------------------------------------------------------------

component IBUFG
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC
    );
end component;

component BUFG
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC
    );
end component;

component IBUF
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC
    );
end component;

component OBUF
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC
    );
end component;

component OBUF_F_12
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC
    );
end component;

component OBUF_S_24
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC
    );
end component;

component OBUFT
  port (
    I : in  STD_LOGIC ;
    O : out STD_LOGIC ;
    T : in  STD_LOGIC
    );
end component;

component IOBUF
  port (
    I  : in    STD_LOGIC ;
    O  : out   STD_LOGIC ;
    IO : inout STD_LOGIC ;
    T  : in    STD_LOGIC 
    );
end component;

component CLKDLL
  port (
    CLK0   : out STD_LOGIC ;
    CLK90  : out STD_LOGIC ;
    CLK180 : out STD_LOGIC ;
    CLK270 : out STD_LOGIC ;
    CLK2X  : out STD_LOGIC ;
    CLKDV  : out STD_LOGIC ;
    LOCKED : out STD_LOGIC ;
    CLKIN  : in  STD_LOGIC ;
    CLKFB  : in  STD_LOGIC ;
    RST    : in  STD_LOGIC
    );
end component;

component STARTUP_VIRTEX
  port (
    GSR : in std_logic;
    GTS : in std_logic;
    CLK : in std_logic
    );
end component;

----------------------------------------------------------------------------
--COMPONENT DECLARATION
----------------------------------------------------------------------------

component fifo_rambs16
  generic (
    width : integer := 1 
    );
  port (
    clk_in         : in  STD_LOGIC;
    rst_in         : in  STD_LOGIC;
    data_in        : in  STD_LOGIC_VECTOR(16*(width-1)+15 DOWNTO 0);
    data_out       : out STD_LOGIC_VECTOR(16*(width-1)+15 DOWNTO 0);
    ren            : in  STD_LOGIC;
    wen            : in  STD_LOGIC;
    full           : out STD_LOGIC;
    empty          : out STD_LOGIC;
    almost_full    : out STD_LOGIC;
    fifo_count_out : out STD_LOGIC_VECTOR( 7 downto 0)
    );
END component ;

component bc_l1_check
  port (
    clk                 : in  STD_LOGIC ;
    rst                 : in  STD_LOGIC ;
    data_in             : in  STD_LOGIC_VECTOR(39 DOWNTO 0) ;
    data_valid_in_n     : in  STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
    form_enabled_n      : in  STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
    data_out            : out STD_LOGIC_VECTOR(46 DOWNTO 0) ;
    data_valid_out      : out STD_LOGIC ;
    ev_fifo_ren         : out STD_LOGIC ;
    ev_fifo_empty       : in  STD_LOGIC ;
    ev_fifo_empty_error : out STD_LOGIC ;
    ev_fifo_data        : in  STD_LOGIC_VECTOR(111 DOWNTO 0);
    gatherer_id         : in  STD_LOGIC ;
    bcid_offset         : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    grp_evt_count_out   : out STD_LOGIC_VECTOR(31 downto 0) ;
    grp_evt_enable_in   : in  STD_LOGIC ;
    l1_id_fifo_data     : out STD_LOGIC_VECTOR(15 downto 0) ;
    l1_id_fifo_addr     : in  STD_LOGIC_VECTOR( 7 DOWNTO 0) ;
    enable_l1_trap_in   : in  STD_LOGIC ;
    l1_trap_full_out    : out STD_LOGIC ;
    latched_l1id        : out STD_LOGIC_VECTOR(3 DOWNTO 0) ;
    latched_bcid        : out STD_LOGIC_VECTOR(7 DOWNTO 0)
    );
END component;

component ev_data_decode
  port (
    clk                   : in  STD_LOGIC ;
    rst                   : in  STD_LOGIC ;
    ev_data_in            : in  STD_LOGIC_VECTOR( 15 DOWNTO 0) ;
    ev_data_wen_n         : in  STD_LOGIC ;
    ev_data_almost_full_n : out STD_LOGIC ;
    ev_data_empty         : out STD_LOGIC ;
    header_ev_id_data     : out STD_LOGIC_VECTOR( 63 DOWNTO 0) ;
    header_ev_id_wen      : out STD_LOGIC ;
    ev_id_data1           : out STD_LOGIC_VECTOR(111 DOWNTO 0) ;
    ev_id_wen1            : out STD_LOGIC ;
    ev_id_data2           : out STD_LOGIC_VECTOR(111 DOWNTO 0) ;
    ev_id_wen2            : out STD_LOGIC ;      
    ev_fifo1_af           : in  STD_LOGIC ;
    ev_fifo2_af           : in  STD_LOGIC ;
    ev_fifo3_af           : in  STD_LOGIC ;
    bcid_offset           : in  STD_LOGIC_VECTOR(7 DOWNTO 0) ;
    bcid_rollover_select  : in  STD_LOGIC
    );
END component ;

component err_detect
  port (
    clk                 : in  STD_LOGIC ;
    rst                 : in  STD_LOGIC ;
    data_in             : in  STD_LOGIC_VECTOR(46 DOWNTO 0) ;
    data_valid_in       : in  STD_LOGIC ;
    data_out            : out STD_LOGIC_VECTOR(45 DOWNTO 0) ;
    data_valid_out      : out STD_LOGIC ;
    err_sumry_fifo_wen  : out STD_LOGIC ;
    err_sumry_fifo_data : out STD_LOGIC_VECTOR(47 DOWNTO 0) ;
    err_mask_wen        : in  STD_LOGIC ;
    err_mask_din        : in  STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    err_mask_dout       : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    err_mask_sel        : in  STD_LOGIC_VECTOR( 5 DOWNTO 0) ;
    mask_bcid_error     : in  STD_LOGIC ;
    mask_l1id_error     : in  STD_LOGIC
    );
END component;

component efb_bus_interface
  port (
    clk                      : in  STD_LOGIC ;
    rst                      : in  STD_LOGIC ;
    err_mask_wen1            : out STD_LOGIC ;
    err_mask_din1            : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    err_mask_dout1           : in  STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    err_mask_sel1            : out STD_LOGIC_VECTOR( 5 DOWNTO 0) ;
    err_mask_wen2            : out STD_LOGIC ;
    err_mask_din2            : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    err_mask_dout2           : in  STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    err_mask_sel2            : out STD_LOGIC_VECTOR( 5 DOWNTO 0) ;
    source_id                : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    format_version           : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    run_number               : out STD_LOGIC_VECTOR(31 DOWNTO 0);
    send_empty_events        : out STD_LOGIC ;
    efb_bus_sel_n            : in  STD_LOGIC ;
    efb_bus_rnw              : in  STD_LOGIC ;
    efb_bus_hwsel            : in  STD_LOGIC;
    efb_bus_addr             : in  STD_LOGIC_VECTOR( 7 DOWNTO 0) ;
    efb_bus_data_in          : in  STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    efb_bus_data_out         : out STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    efb_bus_data_oe_n        : out STD_LOGIC ;
    efb_bus_ack              : out STD_LOGIC ;
    out_fifo_diag_mode       : out STD_LOGIC_VECTOR( 2 DOWNTO 0) ;
    out_fifo1_diag_en        : out STD_LOGIC ;
    out_fifo2_diag_en        : out STD_LOGIC ;
    ht_fifo_diag_en          : out STD_LOGIC ;
    out_fifo_rst             : out STD_LOGIC ;
    out_fifo1_empty_n        : in  STD_LOGIC ;
    out_fifo1_aempty_n       : in  STD_LOGIC ;
    out_fifo1_full_n         : in  STD_LOGIC ;
    out_fifo1_afull_n        : in  STD_LOGIC ;
    out_fifo1_play_done      : in  STD_LOGIC ;
    out_fifo2_empty_n        : in  STD_LOGIC ;
    out_fifo2_aempty_n       : in  STD_LOGIC ;
    out_fifo2_full_n         : in  STD_LOGIC ;
    out_fifo2_afull_n        : in  STD_LOGIC ;
    ht_fifo_empty_n          : in  STD_LOGIC ;
    ht_fifo_full_n           : in  STD_LOGIC ;
    out_fifo2_play_done      : in  STD_LOGIC ;
    out_fifo_data_word       : out STD_LOGIC_VECTOR(45 DOWNTO 0) ;
    out_fifo_write_data      : out STD_LOGIC ;
    out_fifo1_num_words_play : out STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    out_fifo2_num_words_play : out STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    out_fifo_play_data       : out STD_LOGIC ;
    diag_tri_state_out_fifo1 : in  STD_LOGIC ;
    diag_tri_state_out_fifo2 : in  STD_LOGIC ;
    diag_tri_state_ev_header : in  STD_LOGIC ;
    eventmem_status_reg      : in  STD_LOGIC_VECTOR( 7 DOWNTO 0) ;
    efb_misc_stts_reg        : in  STD_LOGIC_VECTOR(30 DOWNTO 0) ;
    ev_header_data           : in  STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    ev_fifo_data1            : in  STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    ev_fifo_data2            : in  STD_LOGIC_VECTOR(15 DOWNTO 0) ;
    mask_bcid_error          : out STD_LOGIC;
    mask_l1id_error          : out STD_LOGIC;
    mask_empty_error         : out STD_LOGIC;
    bcid_offset              : out STD_LOGIC_VECTOR( 7 DOWNTO 0);
    rod_type                 : in  STD_LOGIC;
    link_input_test_sel      : out STD_LOGIC_VECTOR( 3 DOWNTO 0);
    grp_evt_enable_out       : out STD_LOGIC;
    event_count_in           : in  STD_LOGIC_VECTOR(31 downto 0);
    bandwidth_status_in      : in  STD_LOGIC_VECTOR(31 downto 0);
    spare_pins_in            : in  STD_LOGIC_VECTOR(23 DOWNTO 0);
    dll_locked_in            : in  STD_LOGIC;
    mask_boc_clock_err_out   : out STD_LOGIC;
    mask_tim_clock_err_out   : out STD_LOGIC;
    sweeper_event_in         : in  STD_LOGIC;
    sync_event_error_in      : in  STD_LOGIC;
    l1id_fifo_data           : in  STD_LOGIC_VECTOR(31 downto 0);
    l1id_fifo_addr           : out STD_LOGIC_VECTOR( 7 downto 0);
    enable_l1_trap_out       : out STD_LOGIC;
    l1_trap_full_in          : in  STD_LOGIC;
    latched_l1id0            : in  STD_LOGIC_VECTOR( 3 DOWNTO 0);
    latched_l1id1            : in  STD_LOGIC_VECTOR( 3 DOWNTO 0);
    latched_bcid0            : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    latched_bcid1            : in  STD_LOGIC_VECTOR( 7 DOWNTO 0);
    tim_bcid_in_rol_out      : out STD_LOGIC;
    bcid_rollover_select     : out STD_LOGIC;
    hv_flag_set_out          : out STD_LOGIC_VECTOR( 3 DOWNTO 0);
    event_threshold_limit    : out STD_LOGIC_VECTOR(15 downto 0);
    evid_fifo_count_in       : in  STD_LOGIC_VECTOR( 7 downto 0);
    evtmem_fifo_wc_stts_in   : in  STD_LOGIC_VECTOR(21 downto 0);
    diag_fifo_control      : out std_logic_vector(31 downto 0);
    diag_fifo_status       : in  std_logic_vector(31 downto 0);
    diag_fifo_trig_mask    : out std_logic_vector(31 downto 0); 
    diag_fifo_data_in      : in  std_logic_vector(31 downto 0);
    announce_ecr_enable      : out std_logic;
    l1id_err_data           : in  STD_LOGIC_VECTOR(31 downto 0);
    bcid_err_data           : in  STD_LOGIC_VECTOR(31 downto 0);
    desync_cnter_addr       : out STD_LOGIC_VECTOR( 3 downto 0);
    desync_cnter_rst        : out STD_LOGIC
    );
END component;

component format_data
  port (
    clk                 : in  STD_LOGIC ;
    rst                 : in  STD_LOGIC ;
    data_in             : in  STD_LOGIC_VECTOR(45 DOWNTO 0) ;
    data_valid_in       : in  STD_LOGIC ;
    data_out            : out STD_LOGIC_VECTOR(45 DOWNTO 0) ;
    data_valid_out_n    : out STD_LOGIC ;
    count_fifo_wen      : out STD_LOGIC ;
    count_fifo_data     : out STD_LOGIC_VECTOR(13 DOWNTO 0)
    );
END component;

component gen_fragment
  port (
    clk                   : in  STD_LOGIC ;
    rst                   : in  STD_LOGIC ;
    finish_fragment       : in  STD_LOGIC ;
    fast_clear            : in  STD_LOGIC ;
    efb_flush_data        : out STD_LOGIC ;
    send_empty_events     : in  STD_LOGIC ;
    header_ev_id_empty    : in  STD_LOGIC ;
    header_ev_id_data     : in  STD_LOGIC_VECTOR(63 DOWNTO 0) ;
    header_ev_id_ren      : out STD_LOGIC ;
    err_sumry_fifo1_empty : in  STD_LOGIC ;
    err_sumry_fifo1_data  : in  STD_LOGIC_VECTOR(47 DOWNTO 0) ;
    err_sumry_fifo1_ren   : out STD_LOGIC ;
    err_sumry_fifo2_empty : in  STD_LOGIC ;
    err_sumry_fifo2_data  : in  STD_LOGIC_VECTOR(47 DOWNTO 0) ;
    err_sumry_fifo2_ren   : out STD_LOGIC ;
    count_fifo1_empty     : in  STD_LOGIC ;
    count_fifo1_data      : in  STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    count_fifo1_ren       : out STD_LOGIC ;
    count_fifo2_empty     : in  STD_LOGIC ;
    count_fifo2_data      : in  STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    count_fifo2_ren       : out STD_LOGIC ;
    halt_output           : in  STD_LOGIC ;
    out_mem1_play         : out STD_LOGIC ;
    out_mem1_num_words    : out STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    out_mem1_done         : in  STD_LOGIC ;
    out_mem2_play         : out STD_LOGIC ;
    out_mem2_num_words    : out STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    out_mem2_done         : in  STD_LOGIC ;
    ev_header_data_out    : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    ev_header_dv_out_p1   : out STD_LOGIC ;
    source_id             : in  STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    format_version        : in  STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    run_number            : in  STD_LOGIC_VECTOR(31 DOWNTO 0) ;
--    error_sumry_out       : out STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    mask_boc_err_in       : in  STD_LOGIC ;
    mask_tim_err_in       : in  STD_LOGIC ;
    sweeper_event_out     : out STD_LOGIC ;
    sync_event_error_out  : out STD_LOGIC ;
    hold_next_event_in    : in  STD_LOGIC ;
    tim_bcid_in_rol_in    : in  STD_LOGIC ;
    hv_flag_set_in        : in  STD_LOGIC_VECTOR( 3 DOWNTO 0) ;
    event_threshold_limit : in  STD_LOGIC_VECTOR(15 downto 0) ;
    event_id_error_in     : in  STD_LOGIC
    );
END component ;

component outmem 
  port (
    clk                   : in  STD_LOGIC ;
    rst                   : in  STD_LOGIC ;
    xoff                  : in  STD_LOGIC ;
    out_mem_play          : in  STD_LOGIC ;
    out_mem_num_words     : in  STD_LOGIC_VECTOR(13 DOWNTO 0) ;
    out_mem_done          : out STD_LOGIC ;
    out_mem_data_valid_p1 : out STD_LOGIC ;
    out_fifo_ren_n        : out STD_LOGIC ;
    out_fifo_oe_n_p1      : out STD_LOGIC 
    );  
END component;

component spram_256x32
  port (
    clk40_in        : in  std_logic; -- clk40 input
    rst_fifo_in     : in  std_logic; -- fifo reset
    writein_bus_in  : in  std_logic_vector(31 downto 0);
    write_strobe_in : in  std_logic; --
    read_strobe_in  : in  std_logic; --
    readout_bus_out : out std_logic_vector(31 downto 0);
    fifo_empty_out  : out std_logic;
    fifo_full_out   : out std_logic
    );
END component;

--component ringbuf
--  generic(
--    g_bramCount     : integer := 2;
--    g_widthPerBram  : integer := 8
--  );
--  port(
--    IN_clk          : in    std_logic;
--    IN_rst          : in    std_logic;
--    IN_control      : in    std_logic_vector(31 downto 0);
--    OUT_status      : out   std_logic_vector(31 downto 0);
--    IN_trig_mask    : in    std_logic_vector(31 downto 0);
--    IN_triggers     : in    std_logic_vector(31 downto 0);
--    IN_data         : in    std_logic_vector(31 downto 0);
--    IN_valid        : in    std_logic;
--    OUT_data        : out   std_logic_vector(31 downto 0);
--    OUT_valid       : out   std_logic
--  );
--end component;

BEGIN  --  Main Body of VHDL code

----------------------------------------------------------------------------
--COMPONENT INSTANTIATION
----------------------------------------------------------------------------

gnd <= '0';
pwr <= '1';
rst <= NOT rst_n; --jmj 20000623

event_ev_id1 : fifo_rambs16
  generic map(
    width => 7  -- 112 wide
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => ev_id_data_in1,
    data_out       => ev_id_data_out1,  -- ev_id_data_out1(79 downto 64) ROD Event Type
    ren            => ev_id_ren1,
    wen            => ev_id_wen1,
    full           => ev_id_full1,
    empty          => ev_id_empty1,
    almost_full    => ev_id_almost_full1,
    fifo_count_out => open
    );
  
event_ev_id2 : fifo_rambs16
  generic map(
    width => 7  -- 112 wide
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => ev_id_data_in2,
    data_out       => ev_id_data_out2,  -- ev_id_data_out2(79 downto 64) ROD Event Type
    ren            => ev_id_ren2,
    wen            => ev_id_wen2,
    full           => ev_id_full2,
    empty          => ev_id_empty2,
    almost_full    => ev_id_almost_full2,
    fifo_count_out => open
    );

header_ev_id : fifo_rambs16
  generic map(
    width => 4  -- 64 wide
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => header_ev_id_data_in,
    data_out       => header_ev_id_data_out,
    ren            => header_ev_id_ren,
    wen            => header_ev_id_wen,
    full           => header_ev_id_full,
    empty          => header_ev_id_empty,
    almost_full    => header_ev_id_almost_full,
    fifo_count_out => evid_fifo_count_i
    );

gatherer_id1 <= '0';
 
bc_l1_check1: bc_l1_check
  port map (
    clk                 => clk,
    rst                 => rst_or_flush,
    data_in             => data_in1_i,
    data_valid_in_n     => data_valid_in_n1_i,
    form_enabled_n      => form_enabled_n1_i,
    data_out            => id_check1_data_out,
    data_valid_out      => id_check1_data_valid_out,
    ev_fifo_ren         => ev_id_ren1,
    ev_fifo_empty       => ev_id_empty1,
    ev_fifo_empty_error => ev_id_fifo_empty_error1,
    ev_fifo_data        => ev_id_data_out1,
    gatherer_id         => gatherer_id1,
    bcid_offset         => bcid_offset,
    grp_evt_count_out   => grp_evt_count1,
    grp_evt_enable_in   => grp_evt_enable,
    l1_id_fifo_data     => l1id_diag_data(15 downto 0),
    l1_id_fifo_addr     => l1id_diag_addr,
    enable_l1_trap_in   => enable_l1_trap,
    l1_trap_full_out    => l1_trap_full1,
    latched_l1id        => latched_l1id0,
    latched_bcid        => latched_bcid0
    );

gatherer_id2 <= '1';
  
bc_l1_check2: bc_l1_check
  port map (
    clk                 => clk,
    rst                 => rst_or_flush,
    data_in             => data_in2_i,
    data_valid_in_n     => data_valid_in_n2_i,
    form_enabled_n      => form_enabled_n2_i,
    data_out            => id_check2_data_out,
    data_valid_out      => id_check2_data_valid_out,
    ev_fifo_ren         => ev_id_ren2,
    ev_fifo_empty       => ev_id_empty2,
    ev_fifo_empty_error => ev_id_fifo_empty_error2,
    ev_fifo_data        => ev_id_data_out2,
    gatherer_id         => gatherer_id2,
    bcid_offset         => bcid_offset,
    grp_evt_count_out   => grp_evt_count2,
    grp_evt_enable_in   => grp_evt_enable,
    l1_id_fifo_data     => l1id_diag_data(31 downto 16),
    l1_id_fifo_addr     => l1id_diag_addr,
    enable_l1_trap_in   => enable_l1_trap,
    l1_trap_full_out    => l1_trap_full2,
    latched_l1id        => latched_l1id1,
    latched_bcid        => latched_bcid1
    );

l1_trap_full <= l1_trap_full1 AND l1_trap_full2;

event_data_decode: ev_data_decode
  port map (
    clk                   => clk,
    rst                   => rst_or_flush,
    ev_data_in            => ev_data_i,
    ev_data_wen_n         => ev_data_wen_n_i,
    ev_data_almost_full_n => ev_data_almost_full_n_o,
    ev_data_empty         => ev_data_empty_o,
    header_ev_id_data     => header_ev_id_data_in,
    header_ev_id_wen      => header_ev_id_wen,
    ev_id_data1           => ev_id_data_in1,
    ev_id_wen1            => ev_id_wen1,
    ev_id_data2           => ev_id_data_in2,
    ev_id_wen2            => ev_id_wen2,      
    ev_fifo1_af           => header_ev_id_almost_full,
    ev_fifo2_af           => ev_id_almost_full1,
    ev_fifo3_af           => ev_id_almost_full2,
    bcid_offset           => bcid_offset,
    bcid_rollover_select  => bcid_rollover_select
    );

err_detect1: err_detect
  port map( 
    clk                 => clk,
    rst                 => rst_or_flush,
    data_in             => id_check1_data_out,
    data_valid_in       => id_check1_data_valid_out,
    data_out            => err_det1_data_out,
    data_valid_out      => err_det1_data_valid_out,
    err_sumry_fifo_wen  => err_sumry_fifo1_wen,
    err_sumry_fifo_data => err_sumry_fifo1_data_in(47 downto 0),
    err_mask_wen        => err_mask_wen1,
    err_mask_din        => err_mask_din1,
    err_mask_dout       => err_mask_dout1,
    err_mask_sel        => err_mask_sel1,
    mask_bcid_error     => mask_bcid_error,
    mask_l1id_error     => mask_l1id_error
    );

err_detect2: err_detect
  port map( 
    clk                 => clk,
    rst                 => rst_or_flush,
    data_in             => id_check2_data_out,
    data_valid_in       => id_check2_data_valid_out,
    data_out            => err_det2_data_out,
    data_valid_out      => err_det2_data_valid_out,
    err_sumry_fifo_wen  => err_sumry_fifo2_wen,
    err_sumry_fifo_data => err_sumry_fifo2_data_in(47 downto 0),
    err_mask_wen        => err_mask_wen2,
    err_mask_din        => err_mask_din2,
    err_mask_dout       => err_mask_dout2,
    err_mask_sel        => err_mask_sel2,
    mask_bcid_error     => mask_bcid_error,
    mask_l1id_error     => mask_l1id_error
    );

err_sumry_fifo1 : fifo_rambs16
  generic map(
    width => 3  -- 48 bits
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => err_sumry_fifo1_data_in,
    data_out       => err_sumry_fifo1_data_out,
    ren            => err_sumry_fifo1_ren,
    wen            => err_sumry_fifo1_wen,
    full           => err_sumry_fifo1_full,
    empty          => err_sumry_fifo1_empty,
    almost_full    => err_sumry_fifo1_almost_full,
    fifo_count_out => open
    );
  
err_sumry_fifo2 : fifo_rambs16
  generic map(
    width => 3  -- 48 bits
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => err_sumry_fifo2_data_in,
    data_out       => err_sumry_fifo2_data_out,
    ren            => err_sumry_fifo2_ren,
    wen            => err_sumry_fifo2_wen,
    full           => err_sumry_fifo2_full,
    empty          => err_sumry_fifo2_empty,
    almost_full    => err_sumry_fifo2_almost_full,
    fifo_count_out => open
    );

format_data1 : format_data
  port map (
    clk              => clk,
    rst              => rst_or_flush,
    data_in          => err_det1_data_out,
    data_valid_in    => err_det1_data_valid_out,
    data_out         => out_fifo1_data_i,
    data_valid_out_n => out_fifo1_wen_n_i,     
    count_fifo_wen   => count_fifo1_wen,
    count_fifo_data  => count_fifo1_data_in(13 downto 0)
    );

format_data2 : format_data
  port map (
    clk              => clk,
    rst              => rst_or_flush,
    data_in          => err_det2_data_out,
    data_valid_in    => err_det2_data_valid_out,
    data_out         => out_fifo2_data_i,
    data_valid_out_n => out_fifo2_wen_n_i,
    count_fifo_wen   => count_fifo2_wen,
    count_fifo_data  => count_fifo2_data_in(13 downto 0)
    );

ew_count_fifo1 : fifo_rambs16
  generic map(
    width => 1
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => count_fifo1_data_in,
    data_out       => count_fifo1_data_out,
    ren            => count_fifo1_ren,
    wen            => count_fifo1_wen,
    full           => count_fifo1_full,
    empty          => count_fifo1_empty,
    almost_full    => count_fifo1_almost_full,
    fifo_count_out => count_fifo1_wc
    );

  evtmem_fifo_wc_stts_i( 7 downto 0) <= count_fifo1_wc;
  evtmem_fifo_wc_stts_i(10 downto 8) <= count_fifo1_full & count_fifo1_almost_full & count_fifo1_empty;

ew_count_fifo2 : fifo_rambs16
  generic map(
    width => 1
    )
  port map (
    clk_in         => clk,
    rst_in         => rst_or_flush,
    data_in        => count_fifo2_data_in,
    data_out       => count_fifo2_data_out,
    ren            => count_fifo2_ren,
    wen            => count_fifo2_wen,
    full           => count_fifo2_full,
    empty          => count_fifo2_empty,
    almost_full    => count_fifo2_almost_full,
    fifo_count_out => count_fifo2_wc
    );  
  
  evtmem_fifo_wc_stts_i(18 downto 11) <= count_fifo2_wc;
  evtmem_fifo_wc_stts_i(21 downto 19) <= count_fifo2_full & count_fifo2_almost_full & count_fifo2_empty;

event_fragment : gen_fragment
  port map (
    clk                   => clk,
    rst                   => rst,
    finish_fragment       => finish_frag_after_ECR,
    fast_clear            => announce_ecr,
    send_empty_events     => send_empty_events,
    header_ev_id_empty    => header_ev_id_empty,
    header_ev_id_data     => header_ev_id_data_out,
    header_ev_id_ren      => header_ev_id_ren,
    err_sumry_fifo1_empty => err_sumry_fifo1_empty,
    err_sumry_fifo1_data  => err_sumry_fifo1_data_out,
    err_sumry_fifo1_ren   => err_sumry_fifo1_ren,
    err_sumry_fifo2_empty => err_sumry_fifo2_empty,
    err_sumry_fifo2_data  => err_sumry_fifo2_data_out,
    err_sumry_fifo2_ren   => err_sumry_fifo2_ren,
    count_fifo1_empty     => count_fifo1_empty,
    count_fifo1_data      => count_fifo1_data_out(13 downto 0),
    count_fifo1_ren       => count_fifo1_ren,
    count_fifo2_empty     => count_fifo2_empty,
    count_fifo2_data      => count_fifo2_data_out(13 downto 0),
    count_fifo2_ren       => count_fifo2_ren,
    halt_output           => halt_output,
    out_mem1_play         => out_mem1_play,
    out_mem1_num_words    => out_mem1_num_words,
    out_mem1_done         => out_mem1_done,
    out_mem2_play         => out_mem2_play,
    out_mem2_num_words    => out_mem2_num_words,
    out_mem2_done         => out_mem2_done,
    ev_header_data_out    => ev_header_data_i,
    ev_header_dv_out_p1   => ev_header_data_valid_out,
    source_id             => source_id,
    format_version        => format_version,
    run_number            => run_number,
    mask_boc_err_in       => mask_boc_clock_err_i,
    mask_tim_err_in       => mask_tim_clock_err_i,
    sweeper_event_out     => sweeper_event_i,
    sync_event_error_out  => sync_event_error_i,
    hold_next_event_in    => hold_next_event_i,
    tim_bcid_in_rol_in    => tim_bcid_in_rol_i,
    hv_flag_set_in        => hv_flag_set_i,
    event_threshold_limit => event_threshold_limit_i,
    event_id_error_in     => ev_id_fifo_empty_error_o
    );

outmem1_ctrl : outmem 
  port map (
    clk                   => clk,
    rst                   => rst_or_flush,
    xoff                  => halt_output,
    out_mem_play          => out_mem1_play_i,
    out_mem_num_words     => out_mem1_num_words_i,
    out_mem_done          => out_mem1_done,
    out_mem_data_valid_p1 => out_mem1_data_valid,
    out_fifo_ren_n        => out_fifo1_ren_n_i,
    out_fifo_oe_n_p1      => out_fifo1_oe_n_p1
    );   

outmem2_ctrl : outmem 
  port map (
    clk                   => clk,
    rst                   => rst_or_flush,
    xoff                  => halt_output,
    out_mem_play          => out_mem2_play_i,
    out_mem_num_words     => out_mem2_num_words_i,
    out_mem_done          => out_mem2_done,
    out_mem_data_valid_p1 => out_mem2_data_valid,
    out_fifo_ren_n        => out_fifo2_ren_n_i,
    out_fifo_oe_n_p1      => out_fifo2_oe_n_p1
    );  

efb_bus_interface_crtl : efb_bus_interface
  port map (
    clk                      => clk,
    rst                      => rst,
    err_mask_wen1            => err_mask_wen1,
    err_mask_din1            => err_mask_din1,
    err_mask_dout1           => err_mask_dout1, 
    err_mask_sel1            => err_mask_sel1, 
    err_mask_wen2            => err_mask_wen2,
    err_mask_din2            => err_mask_din2, 
    err_mask_dout2           => err_mask_dout2, 
    err_mask_sel2            => err_mask_sel2,
    source_id                => source_id,
    format_version           => format_version,
    run_number               => run_number,
    send_empty_events        => send_empty_events,
    efb_bus_sel_n            => efb_bus_sel_n_i,
    efb_bus_rnw              => efb_bus_rnw_i,
    efb_bus_hwsel            => spare_o(16),  -- efb_hwsel
    efb_bus_addr             => efb_bus_addr_i,
    efb_bus_data_in          => efb_bus_data_in,
    efb_bus_data_out         => efb_bus_data_out,
    efb_bus_data_oe_n        => efb_bus_data_oe_n,
    efb_bus_ack              => efb_bus_ack_o,
    out_fifo_diag_mode       => out_fifo_diag_mode,
    out_fifo1_diag_en        => out_fifo1_diag_en,
    out_fifo2_diag_en        => out_fifo2_diag_en,
    ht_fifo_diag_en          => ht_fifo_diag_en,
    out_fifo_rst             => out_fifo_rst,
    out_fifo1_empty_n        => out_fifo1_empty_n_i,
    out_fifo1_aempty_n       => out_fifo1_aempty_n_i,
    out_fifo1_full_n         => out_fifo1_full_n_i,
    out_fifo1_afull_n        => out_fifo1_afull_n_i,
    out_fifo1_play_done      => out_mem1_done,
    out_fifo2_empty_n        => out_fifo2_empty_n_i,
    out_fifo2_aempty_n       => out_fifo2_aempty_n_i,
    out_fifo2_full_n         => out_fifo2_full_n_i,
    out_fifo2_afull_n        => out_fifo2_afull_n_i,
    ht_fifo_empty_n          => ht_fifo_empty_i,
    ht_fifo_full_n           => ht_fifo_full_i,
    out_fifo2_play_done      => out_mem2_done,
    out_fifo_data_word       => out_fifo_data_word,
    out_fifo_write_data      => out_fifo_write_data,
    out_fifo1_num_words_play => out_fifo1_num_words_play,
    out_fifo2_num_words_play => out_fifo2_num_words_play,
    out_fifo_play_data       => out_fifo_play_data,
    diag_tri_state_out_fifo1 => tri_state_out_fifo1_o,
    diag_tri_state_out_fifo2 => tri_state_out_fifo2_o,
    diag_tri_state_ev_header => tri_state_ev_header_o,
    eventmem_status_reg      => eventmem_status_reg_i,
    efb_misc_stts_reg        => efb_misc_stts_reg_i,
    ev_header_data           => header_ev_id_data_stts_reg,
    ev_fifo_data1            => ev_id_data_out1(15 downto 0),
    ev_fifo_data2            => ev_id_data_out2(15 downto 0),
    mask_bcid_error          => mask_bcid_error,
    mask_l1id_error          => mask_l1id_error,
    mask_empty_error         => mask_empty_error,
    bcid_offset              => bcid_offset,
    rod_type                 => rod_type_o,  -- '0' <==> SCT Version
                                             -- '1' <==> Pixel Version
    link_input_test_sel      => link_in_mux_i,
    grp_evt_enable_out       => grp_evt_enable,
    event_count_in           => event_count_i,
    bandwidth_status_in      => bandwidth_status_i,
    spare_pins_in            => spare_o,
    dll_locked_in            => dll_locked_i,
    mask_boc_clock_err_out   => mask_boc_clock_err_i,
    mask_tim_clock_err_out   => mask_tim_clock_err_i,
    sweeper_event_in         => sweeper_event_i,
    sync_event_error_in      => sync_event_error_i,
    l1id_fifo_data           => l1id_diag_data,
    l1id_fifo_addr           => l1id_diag_addr,
    enable_l1_trap_out       => enable_l1_trap,
    l1_trap_full_in          => l1_trap_full,
    latched_l1id0            => latched_l1id0,
    latched_l1id1            => latched_l1id1,
    latched_bcid0            => latched_bcid0,
    latched_bcid1            => latched_bcid1,
    tim_bcid_in_rol_out      => tim_bcid_in_rol_i,
    bcid_rollover_select     => bcid_rollover_select,
    hv_flag_set_out          => hv_flag_set_i,
    event_threshold_limit    => event_threshold_limit_i,
    evid_fifo_count_in       => evid_fifo_count_i,
    evtmem_fifo_wc_stts_in   => evtmem_fifo_wc_stts_i,
    diag_fifo_control      => diag_fifo_control,
    diag_fifo_status       => diag_fifo_status,
    diag_fifo_trig_mask    => diag_fifo_trig_mask,
    diag_fifo_data_in      => diag_fifo_data_o,
    announce_ecr_enable    => announce_ecr_enable,
    l1id_err_data          => l1id_err_data,
    bcid_err_data          => bcid_err_data,
    desync_cnter_addr      => desync_cnter_addr,
    desync_cnter_rst       => desync_cnter_rst
    );

eventmem_status_reg_i <= eventmem_fifo_cntrl_ii;

header_ev_id_data_stts_reg <= header_ev_id_data_out( 3 downto  0) &
                              header_ev_id_data_out(47 downto 36);

efb_misc_stts_reg_i <=   count_fifo1_full              -- (30)
                       & count_fifo2_full              -- (29)
                       & err_sumry_fifo1_full          -- (28)
                       & err_sumry_fifo2_full          -- (27)
                       & ev_data_empty_o               -- (26)
                       & ev_id_full1                   -- (25)
                       & ev_id_full2                   -- (24)
                       & header_ev_id_full             -- (23)
                       & ev_data_almost_full_n_o       -- (22)
                       & header_ev_id_almost_full      -- (21)
                       & ev_id_almost_full2            -- (20)
                       & ev_id_almost_full1            -- (19)
                       & header_ev_id_data_out(34)     -- (18)
                       & form_enabled_n2_i             -- (17:14)
                       & form_enabled_n1_i             -- (13:10)
                       & gatherer_halt_output_i        -- (9)
                       & fifo1_pause_o                 -- (8)
                       & fifo2_pause_o                 -- (7)
                       & header_ev_id_empty            -- (6)
                       & ev_id_empty2                  -- (5)
                       & ev_id_empty1                  -- (4)
                       & ev_id_fifo_empty_error2       -- (3)
                       & ev_id_fifo_empty_error1       -- (2)
                       & err_sumry_fifo2_almost_full   -- (1)
                       & err_sumry_fifo1_almost_full;  -- (0)


ht_debug_fifo : spram_256x32
  port map (
    clk40_in        => clk,
    rst_fifo_in     => rst_or_flush,
    writein_bus_in  => htf_debug_data_in_i(31 downto 0),
    write_strobe_in => ht_fifo_we_i,
    read_strobe_in  => ht_fifo_re_i,
    readout_bus_out => htf_debug_data_out_i(31 downto 0),
    fifo_empty_out  => ht_fifo_empty_i,
    fifo_full_out   => ht_fifo_full_i
    );

-------------------------------------------------------------------------------
-- Input Link Data Capture FIFO
-------------------------------------------------------------------------------
--U10 : ringbuf
--  generic map(
--    g_bramCount     => 2,
--    g_widthPerBram  => 16
--  )
--  port map(
--    IN_clk          => clk,
--    IN_rst          => rst,
--    IN_control      => diag_fifo_control,
--    OUT_status      => diag_fifo_status,
--    IN_trig_mask    => diag_fifo_trig_mask,
--    IN_triggers     => diag_fifo_triggers,
--    IN_data         => diag_fifo_data_i,
--    IN_valid        => diag_fifo_valid_i,
--    OUT_data        => diag_fifo_data_o,
--    OUT_valid       => open
--  );
--
--diag_fifo_triggers(31 downto 5) <= (others => '0');
----diag_fifo_triggers(31 downto 28) <= decoder_wr_trailer_i(3 downto 0);
----diag_fifo_triggers(27 downto 24) <= link_fifo_ff_i(3 downto 0);
----diag_fifo_triggers(23 downto 20) <= not normal_not_raw_i(3 downto 0);
----diag_fifo_triggers(19 downto 16) <= eoe_word_i(3 downto 0);
----diag_fifo_triggers(15 downto 12) <= time_out_error_i(3 downto 0);
----diag_fifo_triggers(11 downto  8) <= fifo_ren_i(3 downto 0);
----diag_fifo_triggers(7) <= hold_output_in_i;
----diag_fifo_triggers(6) <= not rb_limit_t;
----diag_fifo_triggers(5) <= not ht_limit_t;
--diag_fifo_triggers(4) <= id_check2_data_valid_out;
--diag_fifo_triggers(3) <= id_check2_data_out(42) and id_check2_data_valid_out;
--diag_fifo_triggers(2) <= id_check2_data_out(41) and id_check2_data_valid_out;
--diag_fifo_triggers(1) <= announce_ecr;
--diag_fifo_triggers(0) <= ev_data_flush;

----diag_fifo_data_i(31 downto 30) <= (others => '0');
--diag_fifo_data_i(31)           <= id_check2_data_out(38); -- EFB ENGINE
--diag_fifo_data_i(30)           <= id_check2_data_out(39); -- TIMEOUT ERROR
--diag_fifo_data_i(29)           <= id_check2_data_out(42); -- BCID ERROR
--diag_fifo_data_i(28)           <= id_check2_data_out(41); -- L1ID ERROR
--diag_fifo_data_i(27 downto 26) <= id_check2_data_out(37 downto  36); -- FMT #
--diag_fifo_data_i(25 downto 24) <= id_check2_data_out(33 downto 32); -- Link #

--diag_fifo_data_i(23 downto 20) <= id_check2_data_out(11 downto  8); -- L1ID
---- expt. L1ID
--diag_fifo_data_i(19 downto 16) <= ev_id_data_out2(11 downto 8);

--diag_fifo_data_i(15 downto  8) <= id_check2_data_out(7 downto  0); -- BCID
-- expt. BCID
--diag_fifo_data_i( 7 downto  0) <= ev_id_data_out2(7 downto 0);

--diag_fifo_valid_i <= '1' when id_check2_data_valid_out = '1' and id_check2_data_out(31 downto 29) = "001" else '0';

-- Event Data Process FIFO Flush logic
rst_or_flush <= rst OR ev_data_flush OR out_fifo_rst;
finish_frag_after_ECR <= ev_data_flush OR out_fifo_rst;
-- out_fifo1_rst_n_o <= NOT rst_or_flush;
-- out_fifo2_rst_n_o <= NOT rst_or_flush;

ev_data_rst_gen : process (
  clk, 
  rst
  )
  variable rst_count  : unsigned(7 downto 0);
  variable rst_evtmem : std_logic;
  begin -- 
    if (rst = '1') then
      rst_evtmem := '1';
      out_fifo1_rst_n_o <= '0';
      out_fifo2_rst_n_o <= '0';
      rst_count := X"00";
    elsif (clk'event and clk = '1') then
      if (rst_or_flush = '1') then
        rst_evtmem := '1';
      end if;
      if (rst_evtmem = '1') then
        rst_count := rst_count + 1 ;
        out_fifo1_rst_n_o <= '0';
        out_fifo2_rst_n_o <= '0';
        if (rst_count > 200) then
          rst_evtmem := '0';
        end if;
      else        
        out_fifo1_rst_n_o <= '1';
        out_fifo2_rst_n_o <= '1';
      end if;
    end if;
  end process ev_data_rst_gen;

--
data_valid_out_o  <= ev_header_data_valid_out_i OR 
                     out_mem1_data_valid OR
                     out_mem2_data_valid; 
--
--  bit    signal description
--   0     efb_a_re_n    :  efb_a fifo read enable (active low)
--   1     efb_a_oe_n    :  efb_a fifo output enable (active low)
--   2     efb_a_en_n    :  efb_a fifo bus enable (active low)

mux_outmem1_control_sigs: process (
  out_fifo1_diag_en, 
  out_fifo_play_data,
  out_fifo1_num_words_play, 
  out_mem1_play, 
  out_mem1_num_words,
  eventmem_fifo_cntrl_ii,
  out_fifo_diag_mode,
  out_fifo1_ren_n_i,
  out_fifo1_oe_n_p1,
  rst,
  clk
  )
subtype of1_diag_mode is STD_LOGIC_VECTOR(2 downto 0);
begin
  if (rst = '1') then
    out_mem1_play_i       <= '0';
    out_mem1_num_words_i  <= (others => '0');
    out_fifo1_ren_n_o     <= '1';
    out_fifo1_oe_n_o      <= '1';
    tri_state_out_fifo1_o <= '1';
  else
    case of1_diag_mode(out_fifo_diag_mode) is
      when "001" => 
        out_mem1_play_i       <= out_fifo_play_data AND out_fifo1_diag_en;
        out_mem1_num_words_i  <= out_fifo1_num_words_play;
        out_fifo1_ren_n_o     <= out_fifo1_ren_n_i;
        out_fifo1_oe_n_o      <= out_fifo1_oe_n_p1;
        tri_state_out_fifo1_o <= out_fifo1_oe_n_p1;
      when "010" =>
        out_mem1_play_i       <= '0';
        out_mem1_num_words_i  <= (others => '0');
        out_fifo1_ren_n_o     <= eventmem_fifo_cntrl_ii(0);
        out_fifo1_oe_n_o      <= eventmem_fifo_cntrl_ii(1); 
        tri_state_out_fifo1_o <= eventmem_fifo_cntrl_ii(2);
      when "100" =>
        out_mem1_play_i       <= '0';
        out_mem1_num_words_i  <= (others => '0');
        out_fifo1_ren_n_o     <= eventmem_fifo_cntrl_ii(0);
        out_fifo1_oe_n_o      <= eventmem_fifo_cntrl_ii(1); 
        tri_state_out_fifo1_o <= eventmem_fifo_cntrl_ii(2);
      when others =>
        out_mem1_play_i       <= out_mem1_play;
        out_mem1_num_words_i  <= out_mem1_num_words;
        out_fifo1_ren_n_o     <= out_fifo1_ren_n_i;
        out_fifo1_oe_n_o      <= out_fifo1_oe_n_p1;  --out_fifo1_ren_n_i;
        tri_state_out_fifo1_o <= out_fifo1_oe_n_p1;  --out_fifo1_ren_n_i;
      end case;
  end if;
end process mux_outmem1_control_sigs;

--
--  bit    signal description
--   3     efb_b_re_n    :  efb_b fifo read enable (active low)
--   4     efb_b_oe_n    :  efb_b fifo output enable (active low)
--   5     efb_b_en_n    :  efb_b fifo bus enable (active low)

mux_outmem2_control_sigs: process (
  out_fifo2_diag_en,
  out_fifo_play_data,
  out_fifo2_num_words_play,
  out_mem2_play,
  out_mem2_num_words,
  eventmem_fifo_cntrl_ii,
  out_fifo_diag_mode,
  out_fifo2_ren_n_i,
  out_fifo2_oe_n_p1,
  rst, 
  clk
  )
subtype of2_diag_mode is STD_LOGIC_VECTOR(2 downto 0);
begin
  if (rst = '1') then
    out_mem2_play_i       <= '0';
    out_mem2_num_words_i  <= (others => '0');
    out_fifo2_ren_n_o     <= '1';
    out_fifo2_oe_n_o      <= '1';
    tri_state_out_fifo2_o <= '1';
  else
    case of2_diag_mode(out_fifo_diag_mode) is
      when "001" => 
        out_mem2_play_i       <= out_fifo_play_data AND out_fifo2_diag_en;
        out_mem2_num_words_i  <= out_fifo2_num_words_play;
        out_fifo2_ren_n_o     <= out_fifo2_ren_n_i;
        out_fifo2_oe_n_o      <= out_fifo2_oe_n_p1;
        tri_state_out_fifo2_o <= out_fifo2_oe_n_p1;
      when "010" => 
        out_mem2_play_i       <= '0';
        out_mem2_num_words_i  <= (others => '0');
        out_fifo2_ren_n_o     <= eventmem_fifo_cntrl_ii(3);
        out_fifo2_oe_n_o      <= eventmem_fifo_cntrl_ii(4);
        tri_state_out_fifo2_o <= eventmem_fifo_cntrl_ii(5);
      when "100" => 
        out_mem2_play_i       <= '0';
        out_mem2_num_words_i  <= (others => '0');
        out_fifo2_ren_n_o     <= eventmem_fifo_cntrl_ii(3);
        out_fifo2_oe_n_o      <= eventmem_fifo_cntrl_ii(4);
        tri_state_out_fifo2_o <= eventmem_fifo_cntrl_ii(5);
      when others => 
        out_mem2_play_i       <= out_mem2_play;
        out_mem2_num_words_i  <= out_mem2_num_words;
        out_fifo2_ren_n_o     <= out_fifo2_ren_n_i;
        out_fifo2_oe_n_o      <= out_fifo2_oe_n_p1;  --out_fifo2_ren_n_i;
        tri_state_out_fifo2_o <= out_fifo2_oe_n_p1;  --out_fifo2_ren_n_i;
    end case;
  end if;
end process mux_outmem2_control_sigs;

--
--  bit    signal description

--   6     efb_htf_re_n  :  header fifo read enable (active low)
--   7     efb_htf_en_n  :  header fifo bus enable (active low)

mux_htf_control_sigs: process (
  out_fifo_diag_mode,
  eventmem_fifo_cntrl_i,
  eventmem_fifo_cntrl_ii,
  ev_header_data_valid_out_d1,
  out_fifo_play_data,
  ev_header_data_i,
  ht_fifo_diag_en,
  en_ht_fifo_out_i,
  en_ht_fifo_out_d1,
  htf_debug_data_out_i,
  htf_debug_data_in_i,
  rst, 
  clk
  )
subtype htf_diag_mode is STD_LOGIC_VECTOR(2 downto 0);
begin
  if (rst = '1') then
    ht_fifo_re_i               <= '0';
    tri_state_ev_header_o      <= '1';
    ev_header_data_o           <= (others => '0');
    ev_header_data_valid_out_i <= '0';
  else
    case htf_diag_mode(out_fifo_diag_mode) is
      when "001" => 
        ht_fifo_re_i               <= out_fifo_play_data AND ht_fifo_diag_en;
        tri_state_ev_header_o      <= en_ht_fifo_out_d1;
        ev_header_data_o           <= htf_debug_data_out_i;
        ev_header_data_valid_out_i <= NOT en_ht_fifo_out_d1;
      when "010" => 
        ht_fifo_re_i               <= NOT eventmem_fifo_cntrl_i(6);
        tri_state_ev_header_o      <= eventmem_fifo_cntrl_ii(7);
        ev_header_data_o           <= htf_debug_data_out_i;
        ev_header_data_valid_out_i <= '0';
      when "100" => 
        ht_fifo_re_i               <= NOT eventmem_fifo_cntrl_i(6);
        tri_state_ev_header_o      <= eventmem_fifo_cntrl_ii(7);
        ev_header_data_o           <= htf_debug_data_out_i;
        ev_header_data_valid_out_i <= '0';
      when others => 
        ht_fifo_re_i               <= '0';
        tri_state_ev_header_o      <= NOT ev_header_data_valid_out_d1;    -- jmj 20000623
        ev_header_data_o           <= ev_header_data_i;
        ev_header_data_valid_out_i <= ev_header_data_valid_out_d1;
    end case;
  end if;
end process mux_htf_control_sigs;

register_data : process (
  clk, 
  rst
  )
  subtype select_mode is STD_LOGIC_VECTOR(2 downto 0);
  begin -- register
    if (rst = '1') then
      ev_data_flush <= '0';
      announce_ecr <= '0';
      halt_output   <= '0';     
      fifo1_pause_o <= '0';
      fifo2_pause_o <= '0';
      ev_id_fifo_empty_error_o    <= '0';
      ev_data_almost_full_n_p     <= '1';
      ev_header_data_valid_out_d1 <= '0';
      out_fifo1_data_o   <= (others => '0');
      out_fifo1_wen_n_o  <= '1';
      out_fifo2_data_o   <= (others => '0');
      out_fifo2_wen_n_o  <= '1';
      ht_fifo_we_i       <= '0';
      efb_ready_i        <= '0';
      hold_next_event_i  <= '0';
      xoff_pipeline      <= (others => '0');
      xoff_control_state <= wait_for_xoff;
    elsif (clk'event and clk = '1') then
      ev_data_flush      <= NOT ev_data_rst_n_i;
      hold_next_event_i  <= spare_o(23); -- Input from Router Cal BP && PFIFO EF (This can be removed JMJ 3/2008)
      
      if announce_ecr_enable = '1' then
        announce_ecr       <= not spare_o(19); -- use the not delayed ecr for a fast clear before the delayed one is a hard reset
      else
        announce_ecr       <= '0'; -- use the not delayed ecr for a fast clear before the delayed one is a hard reset
      end if;
      -- buffer some signals going on/off chip

      halt_output <= gatherer_halt_output_i;   
--    Add some Hysteresis to the Xoff Logic  20150925
--      xoff_pipeline <= xoff_pipeline(2 downto 0) & gatherer_halt_output_i;
--      case xoff_control_state is
--        when wait_for_xoff => 
--          halt_output <= '0';
--          halt_delay_count <= (others => '0');
--          if (gatherer_halt_output_i = '1') then
--            halt_output <= '1';
--            xoff_control_state <= halt_efb;
--          end if;
--        when halt_efb => 
--          halt_output <= '1';
--          if (xoff_pipeline = X"0") then
--            halt_delay_count <= halt_delay_count + '1';
--          else
--            halt_delay_count <= (others => '0');
--          end if;
--          if (halt_delay_count = 63) then  -- max 63
--            xoff_control_state <= wait_for_xoff;
--          end if;
--        when others =>
--      end case;

-- 20151023:  Original from EFB-v32
      fifo1_pause_o <= (NOT out_fifo1_afull_n_i)    OR 
                        err_sumry_fifo1_almost_full OR
                        count_fifo1_almost_full;

      fifo2_pause_o <= (NOT out_fifo2_afull_n_i)    OR 
                        err_sumry_fifo2_almost_full OR
                        count_fifo2_almost_full;

-- 20151012:  Check ROD Busy Notification with FIFOs EFB to RCF
--      fifo1_pause_o <= (NOT out_fifo1_afull_n_i)     OR
--                       (NOT ev_data_almost_full_n_o) OR
--                        header_ev_id_almost_full     OR
--                        ev_id_almost_full1           OR
--                        err_sumry_fifo1_almost_full  OR
--                        count_fifo1_almost_full;

--      fifo2_pause_o <= (NOT out_fifo2_afull_n_i)     OR 
--                       (NOT ev_data_almost_full_n_o) OR
--                        header_ev_id_almost_full     OR
--                        ev_id_almost_full2           OR
--                        err_sumry_fifo2_almost_full  OR
--                        count_fifo2_almost_full;
-- 20159027:
      ev_id_fifo_empty_error_o <= (ev_id_fifo_empty_error1 OR ev_id_fifo_empty_error2) AND NOT mask_empty_error;

      ev_data_almost_full_n_p <= ev_data_almost_full_n_o AND NOT (halt_output AND header_ev_id_data_out(34));

      en_ht_fifo_out_i <= NOT (out_fifo_play_data AND ht_fifo_diag_en);
      en_ht_fifo_out_d1 <= en_ht_fifo_out_i;

      ev_header_data_valid_out_d1 <= ev_header_data_valid_out;

      eventmem_fifo_cntrl_ii <= eventmem_fifo_cntrl_i;

      case select_mode(out_fifo_diag_mode) is
        when "001" =>  -- EFB Controls Output FIFO, Write from Registers
          out_fifo1_data_o    <= out_fifo_data_word;
          out_fifo1_wen_n_o   <= NOT (out_fifo_write_data AND out_fifo1_diag_en);
          out_fifo2_data_o    <= out_fifo_data_word;
          out_fifo2_wen_n_o   <= NOT (out_fifo_write_data AND out_fifo2_diag_en);
          ht_fifo_we_i        <= out_fifo_write_data AND ht_fifo_diag_en;
          htf_debug_data_in_i <= out_fifo_data_word(31 downto 0);

        when "010" =>  -- EFB Controls Output FIFO, Write from Registers
          out_fifo1_data_o    <= out_fifo_data_word;
          out_fifo1_wen_n_o   <= NOT (out_fifo_write_data AND out_fifo1_diag_en);
          out_fifo2_data_o    <= out_fifo_data_word;
          out_fifo2_wen_n_o   <= NOT (out_fifo_write_data AND out_fifo2_diag_en);
          ht_fifo_we_i        <= out_fifo_write_data AND ht_fifo_diag_en;
          htf_debug_data_in_i <= out_fifo_data_word(31 downto 0);

        when "100" =>  -- EFB Controls Output FIFO, Write in normal mode
          out_fifo1_data_o    <= out_fifo1_data_i;
          out_fifo1_wen_n_o   <= out_fifo1_wen_n_i; -- OR out_fifo_diag_mode;
          out_fifo2_data_o    <= out_fifo2_data_i; 
          out_fifo2_wen_n_o   <= out_fifo2_wen_n_i; -- OR out_fifo_diag_mode;
          ht_fifo_we_i        <= ev_header_data_valid_out_d1;
          htf_debug_data_in_i <= ev_header_data_i(31 downto 0);

        when others =>
          out_fifo1_data_o    <= out_fifo1_data_i;
          out_fifo1_wen_n_o   <= out_fifo1_wen_n_i; -- OR out_fifo_diag_mode;
          out_fifo2_data_o    <= out_fifo2_data_i; 
          out_fifo2_wen_n_o   <= out_fifo2_wen_n_i; -- OR out_fifo_diag_mode;
          ht_fifo_we_i        <= '0';
          htf_debug_data_in_i <= out_fifo_data_word(31 downto 0);
      end case;
      efb_ready_i <= header_ev_id_wen;
    end if;
  end process register_data;
  
grp_evt_count_mux : process (
  clk, 
  rst
  )
begin -- register
  if (rst = '1') then
    event_count_i <= (others => '0');
  elsif (clk'event and clk = '1') then
    if (grp_evt_enable = '1') then
      if (grp_evt_count1(3 downto 0) = grp_evt_count2(3 downto 0)) then
        event_count_i(3 downto 0) <= grp_evt_count1(3 downto 0);
      end if;
      if (grp_evt_count1(7 downto 4) = grp_evt_count2(7 downto 4)) then
        event_count_i(7 downto 4) <= grp_evt_count1(7 downto 4);
      end if;
      if (grp_evt_count1(11 downto 8) = grp_evt_count2(11 downto 8)) then
        event_count_i(11 downto 8) <= grp_evt_count1(11 downto 8);
      end if;
      if (grp_evt_count1(15 downto 12) = grp_evt_count2(15 downto 12)) then
        event_count_i(15 downto 12) <= grp_evt_count1(15 downto 12);
      end if;
      if (grp_evt_count1(19 downto 16) = grp_evt_count2(19 downto 16)) then
        event_count_i(19 downto 16) <= grp_evt_count1(19 downto 16);
      end if;
      if (grp_evt_count1(23 downto 20) = grp_evt_count2(23 downto 20)) then
        event_count_i(23 downto 20) <= grp_evt_count1(23 downto 20);
      end if;
      if (grp_evt_count1(27 downto 24) = grp_evt_count2(27 downto 24)) then
        event_count_i(27 downto 24) <= grp_evt_count1(27 downto 24);
      end if;
      if (grp_evt_count1(31 downto 28) = grp_evt_count2(31 downto 28)) then
        event_count_i(31 downto 28) <= grp_evt_count1(31 downto 28);
      end if;
    else
      event_count_i <= (others => '0');
    end if;
  end if;
end process grp_evt_count_mux;

bandwidth_counter : process (
  clk, 
  rst
  )
begin -- register
  if (rst = '1') then
    time_count_i <= (others => '0');
  elsif (clk'event and clk = '1') then
    if (time_count_i = "1111111111111111") then
      bw_count_i <= (others => '0');
    elsif (form_enabled_n1_i /= "1111" OR form_enabled_n2_i /= "1111") then
      bw_count_i <= bw_count_i + 1;
    end if;
    time_count_i <= time_count_i + 1;
  end if;
end process bandwidth_counter;
bandwidth_status_i <= time_count_i & bw_count_i;
  
link_in_mux : process (
  link_in_mux_i,
  link_mux_test_p
  )
begin
  case link_in_mux_i is
    when "0000" => link_data_o <= link_mux_test_p(0);
    when "0001" => link_data_o <= link_mux_test_p(1);
    when "0010" => link_data_o <= link_mux_test_p(2);
    when "0011" => link_data_o <= link_mux_test_p(3);
    when "0100" => link_data_o <= link_mux_test_p(4);
    when "0101" => link_data_o <= link_mux_test_p(5);
    when "0110" => link_data_o <= link_mux_test_p(6);
    when "0111" => link_data_o <= link_mux_test_p(7);
    when others => link_data_o <= link_mux_test_p(7) OR
                                  link_mux_test_p(6) OR
                                  link_mux_test_p(5) OR
                                  link_mux_test_p(4) OR
                                  link_mux_test_p(3) OR
                                  link_mux_test_p(2) OR
                                  link_mux_test_p(1) OR
                                  link_mux_test_p(0);

  end case;
end process link_in_mux;


desync_counter: process(clk, rst, desync_cnter_rst)
begin
  if (rst = '1' or desync_cnter_rst = '1') then
    l1id_err_cnter <= (others => (others => '0'));
    bcid_err_cnter <= (others => (others => '0'));
    id_check1_l1id_old <= '0';
    id_check1_bcid_old <= '0';
    id_check2_l1id_old <= '0';
    id_check2_bcid_old <= '0';
  elsif (clk'event and clk = '1') then
    
-- id check output format:      
--      data_out[32:38] link address
--                      [38]    efb engine (0-1)
--                      [37:36] formatter (0-3)
--                      [35:32] link (0-11)
--      data_out[41] L1ID error
--      data_out[42] BCID error
    
--    id_check1_l1id_old <= id_check1_data_out(41);
--    id_check1_bcid_old <= id_check1_data_out(42);
--    id_check2_l1id_old <= id_check2_data_out(41);
--    id_check2_bcid_old <= id_check2_data_out(42);
    
    if id_check1_data_valid_out = '1' then
      id_check1_l1id_old <= id_check1_data_out(41);
      id_check1_bcid_old <= id_check1_data_out(42);
    
      if id_check1_data_out(41) = '1' and id_check1_l1id_old = '0' then
        case id_check1_data_out(37 downto 32) is
          when "000000" =>
            l1id_err_cnter(0) <= l1id_err_cnter(0) + 1;
          when "000001" =>
            l1id_err_cnter(1) <= l1id_err_cnter(1) + 1;
          when "010000" =>
            l1id_err_cnter(2) <= l1id_err_cnter(2) + 1;
          when "010001" =>
            l1id_err_cnter(3) <= l1id_err_cnter(3) + 1;
          when "100000" =>
            l1id_err_cnter(4) <= l1id_err_cnter(4) + 1;
          when "100001" =>
            l1id_err_cnter(5) <= l1id_err_cnter(5) + 1;
          when "110000" =>
            l1id_err_cnter(6) <= l1id_err_cnter(6) + 1;
          when "110001" =>
            l1id_err_cnter(7) <= l1id_err_cnter(7) + 1;
          when others =>
            null;
        end case;
      end if;
      
      if id_check1_data_out(42) = '1' and id_check1_bcid_old = '0' then
        case id_check1_data_out(37 downto 32) is
          when "000000" =>
            bcid_err_cnter(0) <= bcid_err_cnter(0) + 1;
          when "000001" =>
            bcid_err_cnter(1) <= bcid_err_cnter(1) + 1;
          when "010000" =>
            bcid_err_cnter(2) <= bcid_err_cnter(2) + 1;
          when "010001" =>
            bcid_err_cnter(3) <= bcid_err_cnter(3) + 1;
          when "100000" =>
            bcid_err_cnter(4) <= bcid_err_cnter(4) + 1;
          when "100001" =>
            bcid_err_cnter(5) <= bcid_err_cnter(5) + 1;
          when "110000" =>
            bcid_err_cnter(6) <= bcid_err_cnter(6) + 1;
          when "110001" =>
            bcid_err_cnter(7) <= bcid_err_cnter(7) + 1;
          when others =>
            null;
        end case;
      end if;
    end if;
    
    if id_check2_data_valid_out = '1' then
      id_check2_l1id_old <= id_check2_data_out(41);
      id_check2_bcid_old <= id_check2_data_out(42);
    
      if id_check2_data_out(41) = '1' and id_check2_l1id_old = '0' then
        case id_check2_data_out(37 downto 32) is
          when "000000" =>
            l1id_err_cnter( 8) <= l1id_err_cnter( 8) + 1;
          when "000001" =>
            l1id_err_cnter( 9) <= l1id_err_cnter( 9) + 1;
          when "010000" =>
            l1id_err_cnter(10) <= l1id_err_cnter(10) + 1;
          when "010001" =>
            l1id_err_cnter(11) <= l1id_err_cnter(11) + 1;
          when "100000" =>
            l1id_err_cnter(12) <= l1id_err_cnter(12) + 1;
          when "100001" =>
            l1id_err_cnter(13) <= l1id_err_cnter(13) + 1;
          when "110000" =>
            l1id_err_cnter(14) <= l1id_err_cnter(14) + 1;
          when "110001" =>
            l1id_err_cnter(15) <= l1id_err_cnter(15) + 1;
          when others =>
            null;
        end case;
      end if;
      
      if id_check2_data_out(42) = '1' and id_check2_bcid_old = '0' then
        case id_check2_data_out(37 downto 32) is
          when "000000" =>
            bcid_err_cnter( 8) <= bcid_err_cnter( 8) + 1;
          when "000001" =>
            bcid_err_cnter( 9) <= bcid_err_cnter( 9) + 1;
          when "010000" =>
            bcid_err_cnter(10) <= bcid_err_cnter(10) + 1;
          when "010001" =>
            bcid_err_cnter(11) <= bcid_err_cnter(11) + 1;
          when "100000" =>
            bcid_err_cnter(12) <= bcid_err_cnter(12) + 1;
          when "100001" =>
            bcid_err_cnter(13) <= bcid_err_cnter(13) + 1;
          when "110000" =>
            bcid_err_cnter(14) <= bcid_err_cnter(14) + 1;
          when "110001" =>
            bcid_err_cnter(15) <= bcid_err_cnter(15) + 1;
          when others =>
            null;
        end case;
      end if;
    end if;
    
    case desync_cnter_addr is
      when x"0" =>
        l1id_err_data <= l1id_err_cnter( 0);
        bcid_err_data <= bcid_err_cnter( 0);
      when x"1" =>
        l1id_err_data <= l1id_err_cnter( 1);
        bcid_err_data <= bcid_err_cnter( 1);
      when x"2" =>
        l1id_err_data <= l1id_err_cnter( 2);
        bcid_err_data <= bcid_err_cnter( 2);
      when x"3" =>
        l1id_err_data <= l1id_err_cnter( 3);
        bcid_err_data <= bcid_err_cnter( 3);
      when x"4" =>
        l1id_err_data <= l1id_err_cnter( 4);
        bcid_err_data <= bcid_err_cnter( 4);
      when x"5" =>
        l1id_err_data <= l1id_err_cnter( 5);
        bcid_err_data <= bcid_err_cnter( 5);
      when x"6" =>
        l1id_err_data <= l1id_err_cnter( 6);
        bcid_err_data <= bcid_err_cnter( 6);
      when x"7" =>
        l1id_err_data <= l1id_err_cnter( 7);
        bcid_err_data <= bcid_err_cnter( 7);
      when x"8" =>
        l1id_err_data <= l1id_err_cnter( 8);
        bcid_err_data <= bcid_err_cnter( 8);
      when x"9" =>
        l1id_err_data <= l1id_err_cnter( 9);
        bcid_err_data <= bcid_err_cnter( 9);
      when x"A" =>
        l1id_err_data <= l1id_err_cnter(10);
        bcid_err_data <= bcid_err_cnter(10);
      when x"B" =>
        l1id_err_data <= l1id_err_cnter(11);
        bcid_err_data <= bcid_err_cnter(11);
      when x"C" =>
        l1id_err_data <= l1id_err_cnter(12);
        bcid_err_data <= bcid_err_cnter(12);
      when x"D" =>
        l1id_err_data <= l1id_err_cnter(13);
        bcid_err_data <= bcid_err_cnter(13);
      when x"E" =>
        l1id_err_data <= l1id_err_cnter(14);
        bcid_err_data <= bcid_err_cnter(14);
      when x"F" =>
        l1id_err_data <= l1id_err_cnter(15);
        bcid_err_data <= bcid_err_cnter(15);
      when others =>
        l1id_err_data <= (others => '0');
        bcid_err_data <= (others => '0');
    end case;
    
  end if;
end process;

----------------------------------------------
-- I/O Buffer / Clk DLL Instantiation
----------------------------------------------

U1 : STARTUP_VIRTEX
  port map (
    GSR => rst,
    GTS => rst,
    CLK => clk
    );

clk_ibufg: IBUFG 
  port map (I => clock, O => clock_in);

clk_bufg: BUFG 
  port map (I => dll_clk0_out, O => clk);

main_clk_dll: CLKDLL 
  port map (
    CLK0   => dll_clk0_out, 
    CLK90  => open, 
    CLK180 => open,
    CLK270 => open, 
    CLK2X  => open, 
    CLKDV  => open, 
    LOCKED => dll_locked_i,
    CLKIN  => clock_in, 
    CLKFB  => clk, 
    RST    => gnd
    );

reset_ibuf: IBUF 
  port map (I => reset, O => rst_n);    -- jmj 20000623

gen_data_in_block: for i in 0 to 39 generate
  data_in1_ibufs: IBUF 
    port map (I => data_in1(i), O => data_in1_i(i));
  data_in2_ibufs: IBUF 
    port map (I => data_in2(i), O => data_in2_i(i));
end generate;

gen_data_valid_in_n_block: for i in 0 to 3 generate
  data_valid_in_n1_ibufs: IBUF
    port map (I => data_valid_in_n1(i), O => data_valid_in_n1_i(i));

  data_valid_in_n2_ibufs: IBUF
    port map (I => data_valid_in_n2(i), O => data_valid_in_n2_i(i));
end generate;

gen_form_enabled_n_block: for i in 0 to 3 generate
  form_enabled_n1_ibufs: IBUF
    port map (I => form_enabled_n1(i), O => form_enabled_n1_i(i));

  form_enabled_n2_ibufs: IBUF
    port map (I => form_enabled_n2(i), O => form_enabled_n2_i(i));
end generate;

fifo1_pause_obuf: OBUF 
  port map (I => fifo1_pause_o, O => fifo1_pause);

fifo2_pause_obuf: OBUF 
  port map (I => fifo2_pause_o, O => fifo2_pause);

efb_bus_sel_n_ibuf: IBUF 
  port map (I => efb_bus_sel_n, O => efb_bus_sel_n_i);

efb_bus_rnw_ibuf: IBUF 
  port map (I => efb_bus_rnw, O => efb_bus_rnw_i);

efb_bus_addr_block: for i in 0 to 7 generate
  efb_bus_addr_ibufs: IBUF 
    port map (I => efb_bus_addr(i), O => efb_bus_addr_i(i));
end generate;

efb_bus_data_block: for i in 0 to 15 generate
  efb_bus_data_iobufs: IOBUF 
    port map (
      I  => efb_bus_data_out(i), 
      O  => efb_bus_data_in(i),
      IO => efb_bus_data(i), 
      T  => efb_bus_data_oe_n
      );
end generate;

efb_bus_ack_obuf: OBUF 
  port map (I => efb_bus_ack_o, O => efb_bus_ack);

ev_data_block: for i in 0 to 15 generate
  ev_data_ibufs: IBUF 
    port map (I => ev_data(i), O => ev_data_ii(i));
end generate;
ev_data_i <= ev_data_ii(15 downto 1) & (ev_data_ii(0) OR spare_o(17));
--ev_data_i <= ev_data_ii(15 downto 0);

ev_data_wen_n_ibuf: IBUF 
  port map (I => ev_data_wen_n, O => ev_data_wen_n_i);

ev_data_rst_n_ibuf: IBUF 
  port map (I => ev_data_rst_n, O => ev_data_rst_n_i);

ev_data_almost_full_n_obuf: OBUF 
  port map (I => ev_data_almost_full_n_p, O => ev_data_almost_full_n);

ev_data_empty_obuf: OBUF 
  port map (I => efb_ready_i, O => ev_data_empty);
--  port map (I => ev_data_empty_o, O => ev_data_empty);  --
--  port map (I => header_ev_id_empty, O => ev_data_empty);

gatherer_halt_output_ibuf: IBUF 
  port map (I => gatherer_halt_output, O => gatherer_halt_output_i);

out_fifo1_empty_n_ibuf: IBUF 
  port map (I => out_fifo1_empty_n, O => out_fifo1_empty_n_i);

out_fifo1_aempty_n_ibuf: IBUF 
  port map (I => out_fifo1_aempty_n, O => out_fifo1_aempty_n_i);

out_fifo1_full_n_ibuf: IBUF 
  port map (I => out_fifo1_full_n, O => out_fifo1_full_n_i);

out_fifo1_afull_n_ibuf: IBUF 
  port map (I => out_fifo1_afull_n, O => out_fifo1_afull_n_i);

out_fifo2_empty_n_ibuf: IBUF 
  port map (I => out_fifo2_empty_n, O => out_fifo2_empty_n_i);

out_fifo2_aempty_n_ibuf: IBUF 
  port map (I => out_fifo2_aempty_n, O => out_fifo2_aempty_n_i);

out_fifo2_full_n_ibuf: IBUF 
  port map (I => out_fifo2_full_n, O => out_fifo2_full_n_i);

out_fifo2_afull_n_ibuf: IBUF 
  port map (I => out_fifo2_afull_n, O => out_fifo2_afull_n_i);

out_fifo1_wen_n_obuf: OBUF 
  port map (I => out_fifo1_wen_n_o, O => out_fifo1_wen_n);

out_fifo1_rst_n_obuf: OBUF 
  port map (I => out_fifo1_rst_n_o, O => out_fifo1_rst_n);

--out_fifo1_ren_n_obuf: OBUF 
out_fifo1_ren_n_obuf: OBUF_F_12 
  port map (I => out_fifo1_ren_n_o, O => out_fifo1_ren_n);

--out_fifo1_oe_n_obuf: OBUF 
out_fifo1_oe_n_obuf: OBUF_F_12 
  port map (I => out_fifo1_oe_n_o, O => out_fifo1_oe_n);

out_fifo2_wen_n_obuf: OBUF 
  port map (I => out_fifo2_wen_n_o, O => out_fifo2_wen_n);

out_fifo2_rst_n_obuf: OBUF 
  port map (I => out_fifo2_rst_n_o, O => out_fifo2_rst_n);

--out_fifo2_ren_n_obuf: OBUF 
out_fifo2_ren_n_obuf: OBUF_F_12
  port map (I => out_fifo2_ren_n_o, O => out_fifo2_ren_n);

--out_fifo2_oe_n_obuf: OBUF 
out_fifo2_oe_n_obuf: OBUF_F_12 
  port map (I => out_fifo2_oe_n_o, O => out_fifo2_oe_n);

out_fifo1_data_block: for i in 0 to 45 generate
  out_fifo1_data_obuft: OBUF 
    port map (I => out_fifo1_data_o(i), O => out_fifo1_data(i));
end generate;

out_fifo2_data_block: for i in 0 to 45 generate
  out_fifo2_data_obuft: OBUF 
    port map (I => out_fifo2_data_o(i), O => out_fifo2_data(i));
end generate;

ev_header_data_out_block: for i in 0 to 31 generate
  ev_header_data_out_obuft: OBUF 
    port map (I => ev_header_data_o(i), O => ev_header_data(i));
  end generate;

tri_state_out_fifo1_obuf: OBUF 
  port map (I => tri_state_out_fifo1_o, O => tri_state_out_fifo1);
        
tri_state_out_fifo2_obuf: OBUF 
  port map (I => tri_state_out_fifo2_o, O => tri_state_out_fifo2);
        
tri_state_ev_header_obuf: OBUF 
  port map (I => tri_state_ev_header_o, O => tri_state_ev_header);

data_valid_out_obuf: OBUF 
  port map (I => data_valid_out_o, O => data_valid_out);

ev_id_fifo_empty_error_obuf: OBUF 
  port map (I => ev_id_fifo_empty_error_o, O => ev_id_fifo_empty_error);
                                            
eventmem_fifo_cntrl_in_block : for i in 0 to 7 generate
  eventmem_fifo_cntrl_ibuf : IBUF
    port map (I => eventmem_fifo_cntrl(i), O => eventmem_fifo_cntrl_i(i));
  end generate;

rod_type_ibuf : IBUF  --jmj 20021016
  port map (I => rod_type_in, O => rod_type_o);

--------------------------- Spare Pins
-- spares[15: 0] to Formatters
-- spares[19:16] to RCF
--    spare[16] == EFB bus HWSEL bit
--    spare[17] == EvtID Data (0) from RCF
--    spare[19] == EFB ROD Busy bit
-- spares[23:20] to Router
spare_pins_block : for i in 0 to 23 generate
  spare_pins_iobuf : IOBUF
    port map (
      I  => spare_i(i), 
      O  => spare_o(i),
      IO => spare_pin(i),
      T  => spare_t(i)
      );
  end generate;

spare_t(23 downto 20) <= (others => '1');  -- Inputs

--spare_i(19) <= sweeper_event_i;
--spare_t(19) <= '0';  -- Output
--spare_o(19);
spare_t(19) <= '1';  -- Input

spare_i(18) <= ev_data_empty_o;  --efb_ready_i;
spare_t(18) <= '0';  -- Output
spare_t(17 downto  0) <= (others => '1');  -- Inputs

link_mux_inputs_block : for i in 0 to 7 generate
  link_mux_test_inputs_ibuf : IBUF
    port map (I => link_data_mux_in(i), O => link_mux_test_p(i));
  end generate;

link_mux_test_obuf : OBUF_S_24
  port map (I => link_data_o, O => link_data_mux_out);

test_out_obuf : OBUF  --jmj 20010215
  port map (I => data_valid_in_n2_i(0), O => test_out(0));

test_out1_obuf : OBUF  --jmj 20010215
  port map (I => form_enabled_n2_i(0), O => test_out(1));

test_out2_obuf : OBUF  --jmj 20010215
  port map (I => header_ev_id_empty, O => test_out(2));

test_out3_obuf : OBUF  --jmj 20010215
  port map (I => efb_ready_i, O => test_out(3));

test_out4_obuf : OBUF  --jmj 20010215
  port map (I => ev_data_wen_n_i, O => test_out(4));

test_out5_obuf : OBUF  --jmj 20010215     ev_data_rst_n_i
  port map (I => ev_data_almost_full_n_p, O => test_out(5));

test_out6_obuf : OBUF  --jmj 20010215     efb_bus_ack_o
  port map (I => header_ev_id_data_out(34), O => test_out(6));

test_out7_obuf : OBUF  --jmj 20010215
  port map (I => spare_o(23), O => test_out(7));  -- efb_hw_sel

END rtl;
