----------------------------------------------------------------------------
-- LBL - WISCONSIN
-- ATLAS ROD ELECTRONICS
----------------------------------------------------------------------------
-- Filename:
--    dsp_sbsram.vhd
-- Description:
--    ROD dsp sbsram simulator
--       SCT Version
--
-- Produces a data stream to help simulate the DSP sbsram interface
-- 
-- 
----------------------------------------------------------------------------

library IEEE ;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

library UNISIM;
use unisim.all;

----------------------------------------------------------------------------
--PORT DECLARATION
----------------------------------------------------------------------------

entity dsp_sbsram is
port (
  clk40_in : in  std_logic; -- Distributed 40 Mhz clock
  clk80_in : in  std_logic; -- Distributed 80 Mhz clock
  rst_n_in : in  std_logic; -- Powerup Reset

-- DSP signals
  dsp0_int4_in     : in  std_logic;
  dsp0_sbram_me	   : out std_logic;
  dsp0_sbsramclk   : out std_logic;
  dsp0_send_data   : out std_logic;
  dsp0_halt_out	   : out std_logic;
  dsp0_trap_en_out : out std_logic;

  dsp1_int4_in     : in  std_logic;
  dsp1_sbram_me    : out std_logic;
  dsp1_sbsramclk   : out std_logic;
  dsp1_send_data   : out std_logic;
  dsp1_halt_out	   : out std_logic;
  dsp1_trap_en_out : out std_logic;

  dsp2_int4_in     : in  std_logic;
  dsp2_sbram_me    : out std_logic;
  dsp2_sbsramclk   : out std_logic;
  dsp2_send_data   : out std_logic;
  dsp2_halt_out	   : out std_logic;
  dsp2_trap_en_out : out std_logic;

  dsp3_int4_in     : in  std_logic;
  dsp3_sbram_me	   : out std_logic;
  dsp3_sbsramclk   : out std_logic;
  dsp3_send_data   : out std_logic;
  dsp3_halt_out	   : out std_logic; 
  dsp3_trap_en_out : out std_logic
  );
end  dsp_sbsram;

architecture rtl of dsp_sbsram is

----------------------------------------------------------------------------
--SIGNAL DECLARATION
----------------------------------------------------------------------------

signal tsf_ctr : integer; -- index into data bit stream character array
signal rst_n : std_logic; -- internal version of powerup reset signal
signal clk80 : std_logic; -- internal version of clock signal
signal gnd   : std_logic; 

signal dsp0_send_data_n_i : std_logic;
signal dsp1_send_data_n_i : std_logic;
signal dsp2_send_data_n_i : std_logic;
signal dsp3_send_data_n_i : std_logic;

signal dsp0_sbram_me_i : std_logic;
signal dsp1_sbram_me_i : std_logic;
signal dsp2_sbram_me_i : std_logic;
signal dsp3_sbram_me_i : std_logic;

signal dsp0_haltoutput_i : std_logic;
signal dsp1_haltoutput_i : std_logic;
signal dsp2_haltoutput_i : std_logic;
signal dsp3_haltoutput_i : std_logic;

signal dsp0_trap_en_out_i : std_logic;
signal dsp1_trap_en_out_i : std_logic;
signal dsp2_trap_en_out_i : std_logic;
signal dsp3_trap_en_out_i : std_logic;

signal dsp0_ctr: integer;
signal dsp1_ctr: integer;
signal dsp2_ctr: integer;
signal dsp3_ctr: integer;

signal dsp0_int4_i: std_logic;
signal dsp1_int4_i: std_logic;
signal dsp2_int4_i: std_logic;
signal dsp3_int4_i: std_logic;

signal dsp0_int4_latch_i: std_logic;
signal dsp1_int4_latch_i: std_logic;
signal dsp2_int4_latch_i: std_logic;
signal dsp3_int4_latch_i: std_logic;

----------------------------------------------------------------------------
--COMPONENT DECLARATION
----------------------------------------------------------------------------

begin  --  Main Body of VHDL code
----------------------------------------------------------------------------
--COMPONENT INSTANTIATION
----------------------------------------------------------------------------
clk80 <= clk80_in;  
rst_n <= rst_n_in;
gnd <= '0';

dsp0_sbsramclk <= clk80_in;
dsp1_sbsramclk <= clk80_in;
dsp2_sbsramclk <= clk80_in;
dsp3_sbsramclk <= clk80_in; 

dsp0_int4_i <= dsp0_int4_in; 
dsp1_int4_i <= dsp1_int4_in;
dsp2_int4_i <= dsp2_int4_in;
dsp3_int4_i <= dsp3_int4_in;

dsp0_send_data <= dsp0_send_data_n_i;
dsp1_send_data <= dsp1_send_data_n_i;
dsp2_send_data <= dsp2_send_data_n_i;
dsp3_send_data <= dsp3_send_data_n_i;

dsp0_sbram_me <= dsp0_sbram_me_i;
dsp1_sbram_me <= dsp1_sbram_me_i;
dsp2_sbram_me <= dsp2_sbram_me_i;
dsp3_sbram_me <= dsp3_sbram_me_i;

dsp0_halt_out <= dsp0_haltoutput_i;
dsp1_halt_out <= dsp1_haltoutput_i;
dsp2_halt_out <= dsp2_haltoutput_i;
dsp3_halt_out <= dsp3_haltoutput_i;

dsp0_trap_en_out <= dsp0_trap_en_out_i;
dsp1_trap_en_out <= dsp1_trap_en_out_i;
dsp2_trap_en_out <= dsp2_trap_en_out_i;
dsp3_trap_en_out <= dsp3_trap_en_out_i;
------------------------------------------------------------------------------
-- Test Stand: File to Bitstream  
------------------------------------------------------------------------------

dsp_send_data : process (
  clk80, 
  rst_n,
  dsp0_int4_in,
  dsp1_int4_in,
  dsp2_int4_in,
  dsp3_int4_in
  )
begin
  if (rst_n = '0') then
    dsp0_send_data_n_i <= '1';
    dsp0_sbram_me_i <= '1';
    dsp0_haltoutput_i <= '0';
    dsp1_send_data_n_i <= '1';
    dsp1_sbram_me_i <= '1';
    dsp1_haltoutput_i <= '0';
    dsp2_send_data_n_i <= '1';
    dsp2_sbram_me_i <= '1';
    dsp2_haltoutput_i <= '0';
    dsp3_send_data_n_i <= '1';
    dsp3_sbram_me_i <= '1';
    dsp3_haltoutput_i <= '0';
    dsp0_ctr <= 281;
    dsp1_ctr <= 281;
    dsp2_ctr <= 281;
    dsp3_ctr <= 281;
    dsp0_int4_latch_i <= '0';
    dsp1_int4_latch_i <= '0';
    dsp2_int4_latch_i <= '0';
    dsp3_int4_latch_i <= '0';
  elsif (clk80'event and clk80 = '1') then
    dsp0_int4_latch_i <= dsp0_int4_i;
    dsp1_int4_latch_i <= dsp1_int4_i;
    dsp2_int4_latch_i <= dsp2_int4_i;
    dsp3_int4_latch_i <= dsp3_int4_i;


    if (dsp0_int4_latch_i = '0' AND dsp0_int4_i = '1' AND (dsp0_ctr < 0 or dsp0_ctr > 258)) then        dsp0_sbram_me_i <= '0';
      dsp0_haltoutput_i <= '1';
      dsp0_ctr <= 0;
    elsif (dsp0_ctr = 1) then
      dsp0_send_data_n_i <= '0';
      dsp0_ctr <= dsp0_ctr + 1;
    elsif (dsp0_ctr = 256) then
      dsp0_sbram_me_i <= '1';
      dsp0_ctr <= dsp0_ctr + 1;
    elsif (dsp0_ctr = 257) then
      dsp0_send_data_n_i <= '1';
      dsp0_ctr <= dsp0_ctr + 1;
    elsif (dsp0_ctr = 1500) then
      dsp0_haltoutput_i <= '0';
      dsp0_ctr <= dsp0_ctr + 1;
    elsif (dsp0_ctr = 1501) then
      dsp0_haltoutput_i <= '0';
    else
      dsp0_ctr <= dsp0_ctr + 1;   
    end if;

    if (dsp1_int4_latch_i = '0' and  dsp1_int4_i = '1' and (dsp1_ctr < 0 or dsp1_ctr > 258)) then  
      dsp1_ctr <= 0;
      dsp1_sbram_me_i <= '0';
      dsp1_haltoutput_i <= '1';
    elsif (dsp1_ctr = 1) then
      dsp1_send_data_n_i <= '0';
      dsp1_ctr <= dsp1_ctr + 1;
    elsif (dsp1_ctr = 256) then
      dsp1_sbram_me_i <= '1';
      dsp1_ctr <= dsp1_ctr + 1;
    elsif (dsp1_ctr = 257) then
      dsp1_send_data_n_i <= '1';
      dsp1_ctr <= dsp1_ctr + 1;
    elsif (dsp1_ctr = 500) then
      dsp1_haltoutput_i <= '0';
      dsp1_ctr <= dsp1_ctr + 1;
    elsif (dsp1_ctr = 501) then
      dsp1_haltoutput_i <= '0';
    else
      dsp1_ctr <= dsp1_ctr + 1;   
    end if;

    if (dsp2_int4_latch_i = '0'  and dsp2_int4_i = '1' and (dsp2_ctr < 0 or dsp2_ctr > 258)) then  
      dsp2_ctr <= 0;
      dsp2_sbram_me_i <= '0';
      dsp2_haltoutput_i <= '1';
    elsif (dsp2_ctr = 1) then
      dsp2_send_data_n_i <= '0';
      dsp2_ctr <= dsp2_ctr + 1;
    elsif (dsp2_ctr = 256) then
      dsp2_sbram_me_i <= '1';
      dsp2_ctr <= dsp2_ctr + 1;
    elsif (dsp2_ctr = 257) then
      dsp2_send_data_n_i <= '1';
      dsp2_ctr <= dsp2_ctr + 1;
    elsif (dsp2_ctr = 500) then
      dsp2_haltoutput_i <= '0';
      dsp2_ctr <= dsp2_ctr + 1;
    elsif (dsp2_ctr = 501) then
      dsp2_haltoutput_i <= '0';
    else
      dsp2_ctr <= dsp2_ctr + 1;   
    end if;

    if (dsp3_int4_latch_i = '0' and dsp3_int4_i = '1' and (dsp3_ctr < 0 or dsp3_ctr > 258)) then  
      dsp3_ctr <= 0;
      dsp3_sbram_me_i <= '0';
      dsp3_haltoutput_i <= '1';
    elsif (dsp3_ctr = 1) then
      dsp3_send_data_n_i <= '0';
      dsp3_ctr <= dsp3_ctr + 1;
    elsif (dsp3_ctr = 256) then
      dsp3_sbram_me_i <= '1';
      dsp3_ctr <= dsp3_ctr + 1;
    elsif (dsp3_ctr = 257) then
      dsp3_send_data_n_i <= '1';
      dsp3_ctr <= dsp3_ctr + 1;
    elsif (dsp3_ctr = 500) then
      dsp3_haltoutput_i <= '0';
      dsp3_ctr <= dsp3_ctr + 1;
    elsif (dsp3_ctr = 501) then
      dsp3_haltoutput_i <= '0';
    else
      dsp3_ctr <= dsp3_ctr + 1;   
    end if;
  end if;
end process dsp_send_data;

dsp_trap_en_out : process (rst_n)
begin   
  if (rst_n = '0') then
    dsp0_trap_en_out_i <= '0';
    dsp1_trap_en_out_i <= '0';
    dsp2_trap_en_out_i <= '0';   
    dsp3_trap_en_out_i <= '0';
  else
    dsp0_trap_en_out_i <= '1';
    dsp1_trap_en_out_i <= '1';
    dsp2_trap_en_out_i <= '1';   
    dsp3_trap_en_out_i <= '1';
  end if;
end process dsp_trap_en_out;

end rtl;

