----------------------------------------------------------------------------
-- LBL - WISCONSIN
-- ATLAS ROD ELECTRONICS
----------------------------------------------------------------------------
-- Filename:
--    clk_reset_source.vhd
-- Description:
--    ROD clock and reset output source
--       SCT Version
--
-- Generates 40MHz and 80MHz clocks, and the master reset signal
--
----------------------------------------------------------------------------

library IEEE ;
use IEEE.STD_LOGIC_1164.all ;
use IEEE.std_logic_arith.all;

----------------------------------------------------------------------------
--PORT DECLARATION
----------------------------------------------------------------------------

entity clk_reset_source is
port (
    clk40_out : out  STD_LOGIC ; -- Distributed 40 Mhz clock 
    clk80_out : out  STD_LOGIC ; -- Distributed 80 Mhz clock 
    rst_n_out : out  STD_LOGIC ; -- Powerup Reset

    vcc_out	: out  STD_LOGIC ; -- connects to VCC to drive it.
    gnd_out	: out  STD_LOGIC   -- connects to GND to drive it.
    ) ;
end  clk_reset_source;

architecture rtl of clk_reset_source is

----------------------------------------------------------------------------
--SIGNAL DECLARATION
----------------------------------------------------------------------------

  signal rst_n : std_logic := '0'; -- internal powerup reset signal
  signal clk40 : std_logic := '0'; -- internal 40MHz clock signal
  signal clk80 : std_logic := '1'; -- internal 80MHz clock signal
  constant clk40_period : time := 25 ns; -- set system clock period here
 
----------------------------------------------------------------------------
--COMPONENT DECLARATION
----------------------------------------------------------------------------
  
BEGIN  --  Main Body of VHDL code

----------------------------------------------------------------------------
--COMPONENT INSTANTIATION
----------------------------------------------------------------------------

  -- since you can't read an output signal, I used internal versions
  -- that get reassigned here before going to the output.
  clk40_out <= clk40;
  clk80_out <= clk80;
  rst_n_out <= rst_n;

------------------------------------------------------------------------------
-- Test Stand: File to Bitstream  
------------------------------------------------------------------------------
  vcc_out <= '1';
  gnd_out <= '0';

  -- Generate reset pulse and clock signals
  rst_n <= '1' after ((clk40_period * 4) + 1 ns);
  clk40 <= not clk40 after (clk40_period/2);
  clk80 <= not clk80 after (clk40_period/4);
    
END rtl;

